# -*- coding: utf-8 -*-
# ********************************************************************
#
#  $Id: svn_id $
#
#  High-level API for YRelay
#
#  - - - - - - - - - License information: - - - - - - - - -
#
#  Copyright (C) 2011 and beyond by Yoctopuce Sarl, Switzerland.
#
#  Yoctopuce Sarl (hereafter Licensor) grants to you a perpetual
#  non-exclusive license to use, modify, copy and integrate this
#  file into your software for the sole purpose of interfacing
#  with Yoctopuce products.
#
#  You may reproduce and distribute copies of this file in
#  source or object form, as long as the sole purpose of this
#  code is to interface with Yoctopuce products. You must retain
#  this notice in the distributed source file.
#
#  You should refer to Yoctopuce General Terms and Conditions
#  for additional information regarding your rights and
#  obligations.
#
#  THE SOFTWARE AND DOCUMENTATION ARE PROVIDED 'AS IS' WITHOUT
#  WARRANTY OF ANY KIND, EITHER EXPRESS OR IMPLIED, INCLUDING
#  WITHOUT LIMITATION, ANY WARRANTY OF MERCHANTABILITY, FITNESS
#  FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO
#  EVENT SHALL LICENSOR BE LIABLE FOR ANY INCIDENTAL, SPECIAL,
#  INDIRECT OR CONSEQUENTIAL DAMAGES, LOST PROFITS OR LOST DATA,
#  COST OF PROCUREMENT OF SUBSTITUTE GOODS, TECHNOLOGY OR
#  SERVICES, ANY CLAIMS BY THIRD PARTIES (INCLUDING BUT NOT
#  LIMITED TO ANY DEFENSE THEREOF), ANY CLAIMS FOR INDEMNITY OR
#  CONTRIBUTION, OR OTHER SIMILAR COSTS, WHETHER ASSERTED ON THE
#  BASIS OF CONTRACT, TORT (INCLUDING NEGLIGENCE), BREACH OF
#  WARRANTY, OR OTHERWISE.
#
# *********************************************************************
"""
Yoctopuce library: High-level API for YRelay
version: PATCH_WITH_VERSION
requires: yocto_relay_aio
requires: yocto_api
"""
from __future__ import annotations

import sys

# On MicroPython, code below will be wiped out at compile time
if sys.implementation.name != "micropython":
    # In CPython, enable edit-time type checking, including Final declaration
    from typing import Union, Final
    from collections.abc import Callable, Awaitable
    from .yocto_api import _IS_MICROPYTHON, _DYNAMIC_HELPERS
else:
    # In our micropython VM, common generic types are global built-ins
    # Others such as TypeVar should be avoided when using micropython,
    # as they produce overhead in runtime code
    # Final is translated into const() expressions before compilation
    _IS_MICROPYTHON: Final[bool] = True # noqa
    _DYNAMIC_HELPERS: Final[bool] = True # noqa

from .yocto_relay_aio import YRelay as YRelay_aio
from .yocto_api import (
    YAPIContext, YAPI, YFunction, YModule
)

# --- (YRelay class start)
if not _IS_MICROPYTHON:
    # For CPython, use strongly typed callback types
    try:
        YRelayValueCallback = Union[Callable[['YRelay', str], Awaitable[None]], None]
    except TypeError:
        YRelayValueCallback = Union[Callable, Awaitable]
if _IS_MICROPYTHON:
    from collections import namedtuple
    YDelayedPulse = namedtuple("YDelayedPulse", ('target', 'ms', 'moving'))
else:
    from typing import NamedTuple
    class YDelayedPulse(NamedTuple):
        target: int
        ms: int
        moving: int


# noinspection PyProtectedMember
class YRelay(YFunction):
    """
    The YRelay class allows you to drive a Yoctopuce relay or optocoupled output.
    It can be used to simply switch the output on or off, but also to automatically generate short
    pulses of determined duration.
    On devices with two output for each relay (double throw), the two outputs are named A and B,
    with output A corresponding to the idle position (normally closed) and the output B corresponding to the
    active state (normally open).

    """
    _aio: YRelay_aio
    # --- (end of YRelay class start)
    if not _IS_MICROPYTHON:
        # --- (YRelay return codes)
        MAXTIMEONSTATEA_INVALID: Final[int] = YAPI.INVALID_LONG
        MAXTIMEONSTATEB_INVALID: Final[int] = YAPI.INVALID_LONG
        PULSETIMER_INVALID: Final[int] = YAPI.INVALID_LONG
        DELAYEDPULSETIMER_INVALID: Final[None] = None
        COUNTDOWN_INVALID: Final[int] = YAPI.INVALID_LONG
        STATE_A: Final[int] = 0
        STATE_B: Final[int] = 1
        STATE_INVALID: Final[int] = -1
        STATEATPOWERON_UNCHANGED: Final[int] = 0
        STATEATPOWERON_A: Final[int] = 1
        STATEATPOWERON_B: Final[int] = 2
        STATEATPOWERON_INVALID: Final[int] = -1
        OUTPUT_OFF: Final[int] = 0
        OUTPUT_ON: Final[int] = 1
        OUTPUT_INVALID: Final[int] = -1
        # --- (end of YRelay return codes)


    # --- (YRelay implementation)

    @classmethod
    def FirstRelay(cls) -> Union[YRelay, None]:
        """
        Starts the enumeration of relays currently accessible.
        Use the method YRelay.nextRelay() to iterate on
        next relays.

        @return a pointer to a YRelay object, corresponding to
                the first relay currently online, or a None pointer
                if there are none.
        """
        return cls._proxy(cls, YRelay_aio.FirstRelay())

    @classmethod
    def FirstRelayInContext(cls, yctx: YAPIContext) -> Union[YRelay, None]:
        """
        Starts the enumeration of relays currently accessible.
        Use the method YRelay.nextRelay() to iterate on
        next relays.

        @param yctx : a YAPI context.

        @return a pointer to a YRelay object, corresponding to
                the first relay currently online, or a None pointer
                if there are none.
        """
        return cls._proxy(cls, YRelay_aio.FirstRelayInContext(yctx))

    def nextRelay(self):
        """
        Continues the enumeration of relays started using yFirstRelay().
        Caution: You can't make any assumption about the returned relays order.
        If you want to find a specific a relay, use Relay.findRelay()
        and a hardwareID or a logical name.

        @return a pointer to a YRelay object, corresponding to
                a relay currently online, or a None pointer
                if there are no more relays to enumerate.
        """
        return self._proxy(type(self), self._aio.nextRelay())

    if not _DYNAMIC_HELPERS:
        def get_state(self) -> int:
            """
            Returns the state of the relays (A for the idle position, B for the active position).

            @return either YRelay.STATE_A or YRelay.STATE_B, according to the state of the relays (A for the
            idle position, B for the active position)

            On failure, throws an exception or returns YRelay.STATE_INVALID.
            """
            return self._run(self._aio.get_state())

    if not _DYNAMIC_HELPERS:
        def set_state(self, newval: int) -> int:
            """
            Changes the state of the relays (A for the idle position, B for the active position).

            @param newval : either YRelay.STATE_A or YRelay.STATE_B, according to the state of the relays (A
            for the idle position, B for the active position)

            @return YAPI.SUCCESS if the call succeeds.

            On failure, throws an exception or returns a negative error code.
            """
            return self._run(self._aio.set_state(newval))

    if not _DYNAMIC_HELPERS:
        def get_stateAtPowerOn(self) -> int:
            """
            Returns the state of the relays at device startup (A for the idle position,
            B for the active position, UNCHANGED to leave the relay state as is).

            @return a value among YRelay.STATEATPOWERON_UNCHANGED, YRelay.STATEATPOWERON_A and
            YRelay.STATEATPOWERON_B corresponding to the state of the relays at device startup (A for the idle position,
                    B for the active position, UNCHANGED to leave the relay state as is)

            On failure, throws an exception or returns YRelay.STATEATPOWERON_INVALID.
            """
            return self._run(self._aio.get_stateAtPowerOn())

    if not _DYNAMIC_HELPERS:
        def set_stateAtPowerOn(self, newval: int) -> int:
            """
            Changes the state of the relays at device startup (A for the idle position,
            B for the active position, UNCHANGED to leave the relay state as is).
            Remember to call the matching module saveToFlash()
            method, otherwise this call will have no effect.

            @param newval : a value among YRelay.STATEATPOWERON_UNCHANGED, YRelay.STATEATPOWERON_A and
            YRelay.STATEATPOWERON_B corresponding to the state of the relays at device startup (A for the idle position,
                    B for the active position, UNCHANGED to leave the relay state as is)

            @return YAPI.SUCCESS if the call succeeds.

            On failure, throws an exception or returns a negative error code.
            """
            return self._run(self._aio.set_stateAtPowerOn(newval))

    if not _DYNAMIC_HELPERS:
        def get_maxTimeOnStateA(self) -> int:
            """
            Returns the maximum time (ms) allowed for the relay to stay in state
            A before automatically switching back in to B state. Zero means no time limit.

            @return an integer corresponding to the maximum time (ms) allowed for the relay to stay in state
                    A before automatically switching back in to B state

            On failure, throws an exception or returns YRelay.MAXTIMEONSTATEA_INVALID.
            """
            return self._run(self._aio.get_maxTimeOnStateA())

    if not _DYNAMIC_HELPERS:
        def set_maxTimeOnStateA(self, newval: int) -> int:
            """
            Changes the maximum time (ms) allowed for the relay to stay in state A
            before automatically switching back in to B state. Use zero for no time limit.
            Remember to call the saveToFlash()
            method of the module if the modification must be kept.

            @param newval : an integer corresponding to the maximum time (ms) allowed for the relay to stay in state A
                    before automatically switching back in to B state

            @return YAPI.SUCCESS if the call succeeds.

            On failure, throws an exception or returns a negative error code.
            """
            return self._run(self._aio.set_maxTimeOnStateA(newval))

    if not _DYNAMIC_HELPERS:
        def get_maxTimeOnStateB(self) -> int:
            """
            Retourne the maximum time (ms) allowed for the relay to stay in state B
            before automatically switching back in to A state. Zero means no time limit.

            @return an integer

            On failure, throws an exception or returns YRelay.MAXTIMEONSTATEB_INVALID.
            """
            return self._run(self._aio.get_maxTimeOnStateB())

    if not _DYNAMIC_HELPERS:
        def set_maxTimeOnStateB(self, newval: int) -> int:
            """
            Changes the maximum time (ms) allowed for the relay to stay in state B before
            automatically switching back in to A state. Use zero for no time limit.
            Remember to call the saveToFlash()
            method of the module if the modification must be kept.

            @param newval : an integer corresponding to the maximum time (ms) allowed for the relay to stay in
            state B before
                    automatically switching back in to A state

            @return YAPI.SUCCESS if the call succeeds.

            On failure, throws an exception or returns a negative error code.
            """
            return self._run(self._aio.set_maxTimeOnStateB(newval))

    if not _DYNAMIC_HELPERS:
        def get_output(self) -> int:
            """
            Returns the output state of the relays, when used as a simple switch (single throw).

            @return either YRelay.OUTPUT_OFF or YRelay.OUTPUT_ON, according to the output state of the relays,
            when used as a simple switch (single throw)

            On failure, throws an exception or returns YRelay.OUTPUT_INVALID.
            """
            return self._run(self._aio.get_output())

    if not _DYNAMIC_HELPERS:
        def set_output(self, newval: int) -> int:
            """
            Changes the output state of the relays, when used as a simple switch (single throw).

            @param newval : either YRelay.OUTPUT_OFF or YRelay.OUTPUT_ON, according to the output state of the
            relays, when used as a simple switch (single throw)

            @return YAPI.SUCCESS if the call succeeds.

            On failure, throws an exception or returns a negative error code.
            """
            return self._run(self._aio.set_output(newval))

    if not _DYNAMIC_HELPERS:
        def get_pulseTimer(self) -> int:
            """
            Returns the number of milliseconds remaining before the relays is returned to idle position
            (state A), during a measured pulse generation. When there is no ongoing pulse, returns zero.

            @return an integer corresponding to the number of milliseconds remaining before the relays is
            returned to idle position
                    (state A), during a measured pulse generation

            On failure, throws an exception or returns YRelay.PULSETIMER_INVALID.
            """
            return self._run(self._aio.get_pulseTimer())

    if not _DYNAMIC_HELPERS:
        def set_pulseTimer(self, newval: int) -> int:
            return self._run(self._aio.set_pulseTimer(newval))

    if not _DYNAMIC_HELPERS:
        def pulse(self, ms_duration) -> int:
            """
            Sets the relay to output B (active) for a specified duration, then brings it
            automatically back to output A (idle state).

            @param ms_duration : pulse duration, in milliseconds

            @return YAPI.SUCCESS if the call succeeds.

            On failure, throws an exception or returns a negative error code.
            """
            return self._run(self._aio.pulse(ms_duration))

    if not _DYNAMIC_HELPERS:
        def set_delayedPulseTimer(self, newval: YDelayedPulse) -> int:
            return self._run(self._aio.set_delayedPulseTimer(newval))

    if not _DYNAMIC_HELPERS:
        def delayedPulse(self, ms_delay, ms_duration) -> int:
            """
            Schedules a pulse.

            @param ms_delay : waiting time before the pulse, in milliseconds
            @param ms_duration : pulse duration, in milliseconds

            @return YAPI.SUCCESS if the call succeeds.

            On failure, throws an exception or returns a negative error code.
            """
            return self._run(self._aio.delayedPulse(ms_delay, ms_duration))

    if not _DYNAMIC_HELPERS:
        def get_countdown(self) -> int:
            """
            Returns the number of milliseconds remaining before a pulse (delayedPulse() call)
            When there is no scheduled pulse, returns zero.

            @return an integer corresponding to the number of milliseconds remaining before a pulse (delayedPulse() call)
                    When there is no scheduled pulse, returns zero

            On failure, throws an exception or returns YRelay.COUNTDOWN_INVALID.
            """
            return self._run(self._aio.get_countdown())

    @classmethod
    def FindRelay(cls, func: str) -> YRelay:
        """
        Retrieves a relay for a given identifier.
        The identifier can be specified using several formats:

        - FunctionLogicalName
        - ModuleSerialNumber.FunctionIdentifier
        - ModuleSerialNumber.FunctionLogicalName
        - ModuleLogicalName.FunctionIdentifier
        - ModuleLogicalName.FunctionLogicalName


        This function does not require that the relay is online at the time
        it is invoked. The returned object is nevertheless valid.
        Use the method YRelay.isOnline() to test if the relay is
        indeed online at a given time. In case of ambiguity when looking for
        a relay by logical name, no error is notified: the first instance
        found is returned. The search is performed first by hardware name,
        then by logical name.

        If a call to this object's is_online() method returns FALSE although
        you are certain that the matching device is plugged, make sure that you did
        call registerHub() at application initialization time.

        @param func : a string that uniquely characterizes the relay, for instance
                YLTCHRL1.relay1.

        @return a YRelay object allowing you to drive the relay.
        """
        return cls._proxy(cls, YRelay_aio.FindRelay(func))

    @classmethod
    def FindRelayInContext(cls, yctx: YAPIContext, func: str) -> YRelay:
        """
        Retrieves a relay for a given identifier in a YAPI context.
        The identifier can be specified using several formats:

        - FunctionLogicalName
        - ModuleSerialNumber.FunctionIdentifier
        - ModuleSerialNumber.FunctionLogicalName
        - ModuleLogicalName.FunctionIdentifier
        - ModuleLogicalName.FunctionLogicalName


        This function does not require that the relay is online at the time
        it is invoked. The returned object is nevertheless valid.
        Use the method YRelay.isOnline() to test if the relay is
        indeed online at a given time. In case of ambiguity when looking for
        a relay by logical name, no error is notified: the first instance
        found is returned. The search is performed first by hardware name,
        then by logical name.

        @param yctx : a YAPI context
        @param func : a string that uniquely characterizes the relay, for instance
                YLTCHRL1.relay1.

        @return a YRelay object allowing you to drive the relay.
        """
        return cls._proxy(cls, YRelay_aio.FindRelayInContext(yctx, func))

    if not _IS_MICROPYTHON:
        def registerValueCallback(self, callback: YRelayValueCallback) -> int:
            """
            Registers the callback function that is invoked on every change of advertised value.
            The callback is invoked only during the execution of ySleep or yHandleEvents.
            This provides control over the time when the callback is triggered. For good responsiveness, remember to call
            one of these two functions periodically. To unregister a callback, pass a None pointer as argument.

            @param callback : the callback function to call, or a None pointer. The callback function should take two
                    arguments: the function object of which the value has changed, and the character string describing
                    the new advertised value.
            @noreturn
            """
            return super().registerValueCallback(callback)

    if not _DYNAMIC_HELPERS:
        def toggle(self) -> int:
            """
            Switch the relay to the opposite state.

            @return YAPI.SUCCESS if the call succeeds.

            On failure, throws an exception or returns a negative error code.
            """
            return self._run(self._aio.toggle())

    # --- (end of YRelay implementation)




# -*- coding: utf-8 -*-
# ********************************************************************
#
#  $Id: svn_id $
#
#  High-level API for YDualPower
#
#  - - - - - - - - - License information: - - - - - - - - -
#
#  Copyright (C) 2011 and beyond by Yoctopuce Sarl, Switzerland.
#
#  Yoctopuce Sarl (hereafter Licensor) grants to you a perpetual
#  non-exclusive license to use, modify, copy and integrate this
#  file into your software for the sole purpose of interfacing
#  with Yoctopuce products.
#
#  You may reproduce and distribute copies of this file in
#  source or object form, as long as the sole purpose of this
#  code is to interface with Yoctopuce products. You must retain
#  this notice in the distributed source file.
#
#  You should refer to Yoctopuce General Terms and Conditions
#  for additional information regarding your rights and
#  obligations.
#
#  THE SOFTWARE AND DOCUMENTATION ARE PROVIDED 'AS IS' WITHOUT
#  WARRANTY OF ANY KIND, EITHER EXPRESS OR IMPLIED, INCLUDING
#  WITHOUT LIMITATION, ANY WARRANTY OF MERCHANTABILITY, FITNESS
#  FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO
#  EVENT SHALL LICENSOR BE LIABLE FOR ANY INCIDENTAL, SPECIAL,
#  INDIRECT OR CONSEQUENTIAL DAMAGES, LOST PROFITS OR LOST DATA,
#  COST OF PROCUREMENT OF SUBSTITUTE GOODS, TECHNOLOGY OR
#  SERVICES, ANY CLAIMS BY THIRD PARTIES (INCLUDING BUT NOT
#  LIMITED TO ANY DEFENSE THEREOF), ANY CLAIMS FOR INDEMNITY OR
#  CONTRIBUTION, OR OTHER SIMILAR COSTS, WHETHER ASSERTED ON THE
#  BASIS OF CONTRACT, TORT (INCLUDING NEGLIGENCE), BREACH OF
#  WARRANTY, OR OTHERWISE.
#
# *********************************************************************
"""
Yoctopuce library: High-level API for YDualPower
version: PATCH_WITH_VERSION
requires: yocto_dualpower_aio
requires: yocto_api
"""
from __future__ import annotations

import sys

# On MicroPython, code below will be wiped out at compile time
if sys.implementation.name != "micropython":
    # In CPython, enable edit-time type checking, including Final declaration
    from typing import Union, Final
    from collections.abc import Callable, Awaitable
    from .yocto_api import _IS_MICROPYTHON, _DYNAMIC_HELPERS
else:
    # In our micropython VM, common generic types are global built-ins
    # Others such as TypeVar should be avoided when using micropython,
    # as they produce overhead in runtime code
    # Final is translated into const() expressions before compilation
    _IS_MICROPYTHON: Final[bool] = True # noqa
    _DYNAMIC_HELPERS: Final[bool] = True # noqa

from .yocto_dualpower_aio import YDualPower as YDualPower_aio
from .yocto_api import (
    YAPIContext, YAPI, YFunction
)

# --- (YDualPower class start)
if not _IS_MICROPYTHON:
    # For CPython, use strongly typed callback types
    try:
        YDualPowerValueCallback = Union[Callable[['YDualPower', str], Awaitable[None]], None]
    except TypeError:
        YDualPowerValueCallback = Union[Callable, Awaitable]

# noinspection PyProtectedMember
class YDualPower(YFunction):
    """
    The YDualPower class allows you to control
    the power source to use for module functions that require high current.
    The module can also automatically disconnect the external power
    when a voltage drop is observed on the external power source
    (external battery running out of power).

    """
    _aio: YDualPower_aio
    # --- (end of YDualPower class start)
    if not _IS_MICROPYTHON:
        # --- (YDualPower return codes)
        EXTVOLTAGE_INVALID: Final[int] = YAPI.INVALID_UINT
        POWERSTATE_OFF: Final[int] = 0
        POWERSTATE_FROM_USB: Final[int] = 1
        POWERSTATE_FROM_EXT: Final[int] = 2
        POWERSTATE_INVALID: Final[int] = -1
        POWERCONTROL_AUTO: Final[int] = 0
        POWERCONTROL_FROM_USB: Final[int] = 1
        POWERCONTROL_FROM_EXT: Final[int] = 2
        POWERCONTROL_OFF: Final[int] = 3
        POWERCONTROL_INVALID: Final[int] = -1
        # --- (end of YDualPower return codes)


    # --- (YDualPower implementation)

    @classmethod
    def FirstDualPower(cls) -> Union[YDualPower, None]:
        """
        Starts the enumeration of dual power switches currently accessible.
        Use the method YDualPower.nextDualPower() to iterate on
        next dual power switches.

        @return a pointer to a YDualPower object, corresponding to
                the first dual power switch currently online, or a None pointer
                if there are none.
        """
        return cls._proxy(cls, YDualPower_aio.FirstDualPower())

    @classmethod
    def FirstDualPowerInContext(cls, yctx: YAPIContext) -> Union[YDualPower, None]:
        """
        Starts the enumeration of dual power switches currently accessible.
        Use the method YDualPower.nextDualPower() to iterate on
        next dual power switches.

        @param yctx : a YAPI context.

        @return a pointer to a YDualPower object, corresponding to
                the first dual power switch currently online, or a None pointer
                if there are none.
        """
        return cls._proxy(cls, YDualPower_aio.FirstDualPowerInContext(yctx))

    def nextDualPower(self):
        """
        Continues the enumeration of dual power switches started using yFirstDualPower().
        Caution: You can't make any assumption about the returned dual power switches order.
        If you want to find a specific a dual power switch, use DualPower.findDualPower()
        and a hardwareID or a logical name.

        @return a pointer to a YDualPower object, corresponding to
                a dual power switch currently online, or a None pointer
                if there are no more dual power switches to enumerate.
        """
        return self._proxy(type(self), self._aio.nextDualPower())

    if not _DYNAMIC_HELPERS:
        def get_powerState(self) -> int:
            """
            Returns the current power source for module functions that require lots of current.

            @return a value among YDualPower.POWERSTATE_OFF, YDualPower.POWERSTATE_FROM_USB and
            YDualPower.POWERSTATE_FROM_EXT corresponding to the current power source for module functions that
            require lots of current

            On failure, throws an exception or returns YDualPower.POWERSTATE_INVALID.
            """
            return self._run(self._aio.get_powerState())

    if not _DYNAMIC_HELPERS:
        def get_powerControl(self) -> int:
            """
            Returns the selected power source for module functions that require lots of current.

            @return a value among YDualPower.POWERCONTROL_AUTO, YDualPower.POWERCONTROL_FROM_USB,
            YDualPower.POWERCONTROL_FROM_EXT and YDualPower.POWERCONTROL_OFF corresponding to the selected
            power source for module functions that require lots of current

            On failure, throws an exception or returns YDualPower.POWERCONTROL_INVALID.
            """
            return self._run(self._aio.get_powerControl())

    if not _DYNAMIC_HELPERS:
        def set_powerControl(self, newval: int) -> int:
            """
            Changes the selected power source for module functions that require lots of current.
            Remember to call the saveToFlash() method of the module if the modification must be kept.

            @param newval : a value among YDualPower.POWERCONTROL_AUTO, YDualPower.POWERCONTROL_FROM_USB,
            YDualPower.POWERCONTROL_FROM_EXT and YDualPower.POWERCONTROL_OFF corresponding to the selected
            power source for module functions that require lots of current

            @return YAPI.SUCCESS if the call succeeds.

            On failure, throws an exception or returns a negative error code.
            """
            return self._run(self._aio.set_powerControl(newval))

    if not _DYNAMIC_HELPERS:
        def get_extVoltage(self) -> int:
            """
            Returns the measured voltage on the external power source, in millivolts.

            @return an integer corresponding to the measured voltage on the external power source, in millivolts

            On failure, throws an exception or returns YDualPower.EXTVOLTAGE_INVALID.
            """
            return self._run(self._aio.get_extVoltage())

    @classmethod
    def FindDualPower(cls, func: str) -> YDualPower:
        """
        Retrieves a dual power switch for a given identifier.
        The identifier can be specified using several formats:

        - FunctionLogicalName
        - ModuleSerialNumber.FunctionIdentifier
        - ModuleSerialNumber.FunctionLogicalName
        - ModuleLogicalName.FunctionIdentifier
        - ModuleLogicalName.FunctionLogicalName


        This function does not require that the dual power switch is online at the time
        it is invoked. The returned object is nevertheless valid.
        Use the method YDualPower.isOnline() to test if the dual power switch is
        indeed online at a given time. In case of ambiguity when looking for
        a dual power switch by logical name, no error is notified: the first instance
        found is returned. The search is performed first by hardware name,
        then by logical name.

        If a call to this object's is_online() method returns FALSE although
        you are certain that the matching device is plugged, make sure that you did
        call registerHub() at application initialization time.

        @param func : a string that uniquely characterizes the dual power switch, for instance
                SERVORC1.dualPower.

        @return a YDualPower object allowing you to drive the dual power switch.
        """
        return cls._proxy(cls, YDualPower_aio.FindDualPower(func))

    @classmethod
    def FindDualPowerInContext(cls, yctx: YAPIContext, func: str) -> YDualPower:
        """
        Retrieves a dual power switch for a given identifier in a YAPI context.
        The identifier can be specified using several formats:

        - FunctionLogicalName
        - ModuleSerialNumber.FunctionIdentifier
        - ModuleSerialNumber.FunctionLogicalName
        - ModuleLogicalName.FunctionIdentifier
        - ModuleLogicalName.FunctionLogicalName


        This function does not require that the dual power switch is online at the time
        it is invoked. The returned object is nevertheless valid.
        Use the method YDualPower.isOnline() to test if the dual power switch is
        indeed online at a given time. In case of ambiguity when looking for
        a dual power switch by logical name, no error is notified: the first instance
        found is returned. The search is performed first by hardware name,
        then by logical name.

        @param yctx : a YAPI context
        @param func : a string that uniquely characterizes the dual power switch, for instance
                SERVORC1.dualPower.

        @return a YDualPower object allowing you to drive the dual power switch.
        """
        return cls._proxy(cls, YDualPower_aio.FindDualPowerInContext(yctx, func))

    if not _IS_MICROPYTHON:
        def registerValueCallback(self, callback: YDualPowerValueCallback) -> int:
            """
            Registers the callback function that is invoked on every change of advertised value.
            The callback is invoked only during the execution of ySleep or yHandleEvents.
            This provides control over the time when the callback is triggered. For good responsiveness, remember to call
            one of these two functions periodically. To unregister a callback, pass a None pointer as argument.

            @param callback : the callback function to call, or a None pointer. The callback function should take two
                    arguments: the function object of which the value has changed, and the character string describing
                    the new advertised value.
            @noreturn
            """
            return super().registerValueCallback(callback)

    # --- (end of YDualPower implementation)




# -*- coding: utf-8 -*-
# ********************************************************************
#
#  $Id: yocto_wireless_aio.py 66072 2025-04-30 06:59:12Z mvuilleu $
#
#  Implements the asyncio YWireless API for Wireless functions
#
#  - - - - - - - - - License information: - - - - - - - - -
#
#  Copyright (C) 2011 and beyond by Yoctopuce Sarl, Switzerland.
#
#  Yoctopuce Sarl (hereafter Licensor) grants to you a perpetual
#  non-exclusive license to use, modify, copy and integrate this
#  file into your software for the sole purpose of interfacing
#  with Yoctopuce products.
#
#  You may reproduce and distribute copies of this file in
#  source or object form, as long as the sole purpose of this
#  code is to interface with Yoctopuce products. You must retain
#  this notice in the distributed source file.
#
#  You should refer to Yoctopuce General Terms and Conditions
#  for additional information regarding your rights and
#  obligations.
#
#  THE SOFTWARE AND DOCUMENTATION ARE PROVIDED 'AS IS' WITHOUT
#  WARRANTY OF ANY KIND, EITHER EXPRESS OR IMPLIED, INCLUDING
#  WITHOUT LIMITATION, ANY WARRANTY OF MERCHANTABILITY, FITNESS
#  FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO
#  EVENT SHALL LICENSOR BE LIABLE FOR ANY INCIDENTAL, SPECIAL,
#  INDIRECT OR CONSEQUENTIAL DAMAGES, LOST PROFITS OR LOST DATA,
#  COST OF PROCUREMENT OF SUBSTITUTE GOODS, TECHNOLOGY OR
#  SERVICES, ANY CLAIMS BY THIRD PARTIES (INCLUDING BUT NOT
#  LIMITED TO ANY DEFENSE THEREOF), ANY CLAIMS FOR INDEMNITY OR
#  CONTRIBUTION, OR OTHER SIMILAR COSTS, WHETHER ASSERTED ON THE
#  BASIS OF CONTRACT, TORT (INCLUDING NEGLIGENCE), BREACH OF
#  WARRANTY, OR OTHERWISE.
#
# *********************************************************************
"""
Yoctopuce library: Asyncio implementation of YCellular
version: PATCH_WITH_VERSION
requires: yocto_api_aio
"""
from __future__ import annotations

import sys

# On MicroPython, code below will be wiped out at compile time
if sys.implementation.name != "micropython":
    # In CPython, enable edit-time type checking, including Final declaration
    from typing import Any, Union, Final
    from collections.abc import Callable, Awaitable
    from .yocto_api_aio import const, _IS_MICROPYTHON
else:
    # In our micropython VM, common generic types are global built-ins
    # Others such as TypeVar should be avoided when using micropython,
    # as they produce overhead in runtime code
    # Final is translated into const() expressions before compilation
    _IS_MICROPYTHON: Final[bool] = True # noqa

from .yocto_api_aio import (
    YAPIContext, YAPI, YAPI_Exception, YFunction, HwId, hwid2str, YModule
)

# --- (generated code: YWlanRecord class start)
# noinspection PyProtectedMember
class YWlanRecord:
    """
    YWlanRecord objects are used to describe a wireless network.
    These objects are  used in particular in conjunction with the
    YWireless class.

    """
    # --- (end of generated code: YWlanRecord class start)
    if not _IS_MICROPYTHON:
        # --- (generated code: YWlanRecord return codes)
        pass
        # --- (end of generated code: YWlanRecord return codes)
        pass

    # --- (generated code: YWlanRecord attributes declaration)
    _ssid: str
    _channel: int
    _sec: str
    _rssi: int
    # --- (end of generated code: YWlanRecord attributes declaration)

    def __init__(self, json_data: XStringIO):
        # --- (generated code: YWlanRecord constructor)
        self._ssid = ''
        self._channel = 0
        self._sec = ''
        self._rssi = 0
        # --- (end of generated code: YWlanRecord constructor)
        new_json: Any = json.load(json_data)
        self._ssid = new_json["ssid"]
        self._channel = new_json["channel"]
        self._sec = new_json["sec"]
        self._rssi = new_json["rssi"]

    # --- (generated code: YWlanRecord implementation)
    def get_ssid(self) -> str:
        """
        Returns the name of the wireless network (SSID).

        @return a string with the name of the wireless network (SSID).
        """
        return self._ssid

    def get_channel(self) -> int:
        """
        Returns the 802.11 b/g/n channel number used by this network.

        @return an integer corresponding to the channel.
        """
        return self._channel

    def get_security(self) -> str:
        """
        Returns the security algorithm used by the wireless network.
        If the network implements to security, the value is "OPEN".

        @return a string with the security algorithm.
        """
        return self._sec

    def get_linkQuality(self) -> int:
        """
        Returns the quality of the wireless network link, in per cents.

        @return an integer between 0 and 100 corresponding to the signal quality.
        """
        return self._rssi

    # --- (end of generated code: YWlanRecord implementation)


# --- (generated code: YWireless class start)
if not _IS_MICROPYTHON:
    # For CPython, use strongly typed callback types
    try:
        YWirelessValueCallback = Union[Callable[['YWireless', str], Awaitable[None]], None]
    except TypeError:
        YWirelessValueCallback = Union[Callable, Awaitable]

# noinspection PyProtectedMember
class YWireless(YFunction):
    """
    The YWireless class provides control over wireless network parameters
    and status for devices that are wireless-enabled.
    Note that TCP/IP parameters are configured separately, using class YNetwork.

    """
    # --- (end of generated code: YWireless class start)
    if not _IS_MICROPYTHON:
        # --- (generated code: YWireless return codes)
        LINKQUALITY_INVALID: Final[int] = YAPI.INVALID_UINT
        SSID_INVALID: Final[str] = YAPI.INVALID_STRING
        CHANNEL_INVALID: Final[int] = YAPI.INVALID_UINT
        MESSAGE_INVALID: Final[str] = YAPI.INVALID_STRING
        WLANCONFIG_INVALID: Final[str] = YAPI.INVALID_STRING
        SECURITY_UNKNOWN: Final[int] = 0
        SECURITY_OPEN: Final[int] = 1
        SECURITY_WEP: Final[int] = 2
        SECURITY_WPA: Final[int] = 3
        SECURITY_WPA2: Final[int] = 4
        SECURITY_INVALID: Final[int] = -1
        WLANSTATE_DOWN: Final[int] = 0
        WLANSTATE_SCANNING: Final[int] = 1
        WLANSTATE_CONNECTED: Final[int] = 2
        WLANSTATE_REJECTED: Final[int] = 3
        WLANSTATE_INVALID: Final[int] = -1
        # --- (end of generated code: YWireless return codes)

    # --- (generated code: YWireless attributes declaration)
    _linkQuality: int
    _ssid: str
    _channel: int
    _security: int
    _message: str
    _wlanConfig: str
    _wlanState: int
    _valueCallback: YWirelessValueCallback
    # --- (end of generated code: YWireless attributes declaration)

    def __init__(self, yctx: YAPIContext, func: str):
        super().__init__(yctx, func)
        self._className = 'Wireless'
        # --- (generated code: YWireless constructor)
        self._linkQuality = YWireless.LINKQUALITY_INVALID
        self._ssid = YWireless.SSID_INVALID
        self._channel = YWireless.CHANNEL_INVALID
        self._security = YWireless.SECURITY_INVALID
        self._message = YWireless.MESSAGE_INVALID
        self._wlanConfig = YWireless.WLANCONFIG_INVALID
        self._wlanState = YWireless.WLANSTATE_INVALID
        # --- (end of generated code: YWireless constructor)

    # --- (generated code: YWireless implementation)

    @staticmethod
    def FirstWireless() -> Union[YWireless, None]:
        """
        Starts the enumeration of wireless LAN interfaces currently accessible.
        Use the method YWireless.nextWireless() to iterate on
        next wireless LAN interfaces.

        @return a pointer to a YWireless object, corresponding to
                the first wireless LAN interface currently online, or a None pointer
                if there are none.
        """
        next_hwid: Union[HwId, None] = YAPI._yHash.getFirstHardwareId('Wireless')
        if not next_hwid:
            return None
        return YWireless.FindWireless(hwid2str(next_hwid))

    @staticmethod
    def FirstWirelessInContext(yctx: YAPIContext) -> Union[YWireless, None]:
        """
        Starts the enumeration of wireless LAN interfaces currently accessible.
        Use the method YWireless.nextWireless() to iterate on
        next wireless LAN interfaces.

        @param yctx : a YAPI context.

        @return a pointer to a YWireless object, corresponding to
                the first wireless LAN interface currently online, or a None pointer
                if there are none.
        """
        next_hwid: Union[HwId, None] = yctx._yHash.getFirstHardwareId('Wireless')
        if not next_hwid:
            return None
        return YWireless.FindWirelessInContext(yctx, hwid2str(next_hwid))

    def nextWireless(self):
        """
        Continues the enumeration of wireless LAN interfaces started using yFirstWireless().
        Caution: You can't make any assumption about the returned wireless LAN interfaces order.
        If you want to find a specific a wireless LAN interface, use Wireless.findWireless()
        and a hardwareID or a logical name.

        @return a pointer to a YWireless object, corresponding to
                a wireless LAN interface currently online, or a None pointer
                if there are no more wireless LAN interfaces to enumerate.
        """
        next_hwid: Union[HwId, None] = None
        try:
            hwid: HwId = self._yapi._yHash.resolveHwID(self._className, self._func)
            next_hwid = self._yapi._yHash.getNextHardwareId(self._className, hwid)
        except YAPI_Exception:
            pass
        if not next_hwid:
            return None
        return YWireless.FindWirelessInContext(self._yapi, hwid2str(next_hwid))

    def _parseAttr(self, json_val: dict) -> None:
        if 'linkQuality' in json_val:
            self._linkQuality = json_val["linkQuality"]
        if 'ssid' in json_val:
            self._ssid = json_val["ssid"]
        if 'channel' in json_val:
            self._channel = json_val["channel"]
        if 'security' in json_val:
            self._security = json_val["security"]
        if 'message' in json_val:
            self._message = json_val["message"]
        if 'wlanConfig' in json_val:
            self._wlanConfig = json_val["wlanConfig"]
        if 'wlanState' in json_val:
            self._wlanState = json_val["wlanState"]
        super()._parseAttr(json_val)

    async def get_linkQuality(self) -> int:
        """
        Returns the link quality, expressed in percent.

        @return an integer corresponding to the link quality, expressed in percent

        On failure, throws an exception or returns YWireless.LINKQUALITY_INVALID.
        """
        res: int
        if self._cacheExpiration <= YAPI.GetTickCount():
            if await self.load(self._yapi.GetCacheValidity()) != YAPI.SUCCESS:
                return YWireless.LINKQUALITY_INVALID
        res = self._linkQuality
        return res

    async def get_ssid(self) -> str:
        """
        Returns the wireless network name (SSID).

        @return a string corresponding to the wireless network name (SSID)

        On failure, throws an exception or returns YWireless.SSID_INVALID.
        """
        res: str
        if self._cacheExpiration <= YAPI.GetTickCount():
            if await self.load(self._yapi.GetCacheValidity()) != YAPI.SUCCESS:
                return YWireless.SSID_INVALID
        res = self._ssid
        return res

    async def get_channel(self) -> int:
        """
        Returns the 802.11 channel currently used, or 0 when the selected network has not been found.

        @return an integer corresponding to the 802.11 channel currently used, or 0 when the selected
        network has not been found

        On failure, throws an exception or returns YWireless.CHANNEL_INVALID.
        """
        res: int
        if self._cacheExpiration <= YAPI.GetTickCount():
            if await self.load(self._yapi.GetCacheValidity()) != YAPI.SUCCESS:
                return YWireless.CHANNEL_INVALID
        res = self._channel
        return res

    async def get_security(self) -> int:
        """
        Returns the security algorithm used by the selected wireless network.

        @return a value among YWireless.SECURITY_UNKNOWN, YWireless.SECURITY_OPEN, YWireless.SECURITY_WEP,
        YWireless.SECURITY_WPA and YWireless.SECURITY_WPA2 corresponding to the security algorithm used by
        the selected wireless network

        On failure, throws an exception or returns YWireless.SECURITY_INVALID.
        """
        res: int
        if self._cacheExpiration <= YAPI.GetTickCount():
            if await self.load(self._yapi.GetCacheValidity()) != YAPI.SUCCESS:
                return YWireless.SECURITY_INVALID
        res = self._security
        return res

    async def get_message(self) -> str:
        """
        Returns the latest status message from the wireless interface.

        @return a string corresponding to the latest status message from the wireless interface

        On failure, throws an exception or returns YWireless.MESSAGE_INVALID.
        """
        res: str
        if self._cacheExpiration <= YAPI.GetTickCount():
            if await self.load(self._yapi.GetCacheValidity()) != YAPI.SUCCESS:
                return YWireless.MESSAGE_INVALID
        res = self._message
        return res

    async def get_wlanConfig(self) -> str:
        res: str
        if self._cacheExpiration <= YAPI.GetTickCount():
            if await self.load(self._yapi.GetCacheValidity()) != YAPI.SUCCESS:
                return YWireless.WLANCONFIG_INVALID
        res = self._wlanConfig
        return res

    async def set_wlanConfig(self, newval: str) -> int:
        rest_val = newval
        return await self._setAttr("wlanConfig", rest_val)

    async def get_wlanState(self) -> int:
        """
        Returns the current state of the wireless interface. The state YWireless.WLANSTATE_DOWN means that
        the network interface is
        not connected to a network. The state YWireless.WLANSTATE_SCANNING means that the network interface
        is scanning available
        frequencies. During this stage, the device is not reachable, and the network settings are not yet
        applied. The state
        YWireless.WLANSTATE_CONNECTED means that the network settings have been successfully applied ant
        that the device is reachable
        from the wireless network. If the device is configured to use ad-hoc or Soft AP mode, it means that
        the wireless network
        is up and that other devices can join the network. The state YWireless.WLANSTATE_REJECTED means
        that the network interface has
        not been able to join the requested network. The description of the error can be obtain with the
        get_message() method.

        @return a value among YWireless.WLANSTATE_DOWN, YWireless.WLANSTATE_SCANNING,
        YWireless.WLANSTATE_CONNECTED and YWireless.WLANSTATE_REJECTED corresponding to the current state
        of the wireless interface

        On failure, throws an exception or returns YWireless.WLANSTATE_INVALID.
        """
        res: int
        if self._cacheExpiration <= YAPI.GetTickCount():
            if await self.load(self._yapi.GetCacheValidity()) != YAPI.SUCCESS:
                return YWireless.WLANSTATE_INVALID
        res = self._wlanState
        return res

    @staticmethod
    def FindWireless(func: str) -> YWireless:
        """
        Retrieves a wireless LAN interface for a given identifier.
        The identifier can be specified using several formats:

        - FunctionLogicalName
        - ModuleSerialNumber.FunctionIdentifier
        - ModuleSerialNumber.FunctionLogicalName
        - ModuleLogicalName.FunctionIdentifier
        - ModuleLogicalName.FunctionLogicalName


        This function does not require that the wireless LAN interface is online at the time
        it is invoked. The returned object is nevertheless valid.
        Use the method YWireless.isOnline() to test if the wireless LAN interface is
        indeed online at a given time. In case of ambiguity when looking for
        a wireless LAN interface by logical name, no error is notified: the first instance
        found is returned. The search is performed first by hardware name,
        then by logical name.

        If a call to this object's is_online() method returns FALSE although
        you are certain that the matching device is plugged, make sure that you did
        call registerHub() at application initialization time.

        @param func : a string that uniquely characterizes the wireless LAN interface, for instance
                YHUBWLN1.wireless.

        @return a YWireless object allowing you to drive the wireless LAN interface.
        """
        obj: Union[YWireless, None]
        obj = YFunction._FindFromCache("Wireless", func)
        if obj is None:
            obj = YWireless(YAPI, func)
            YFunction._AddToCache("Wireless", func, obj)
        return obj

    @staticmethod
    def FindWirelessInContext(yctx: YAPIContext, func: str) -> YWireless:
        """
        Retrieves a wireless LAN interface for a given identifier in a YAPI context.
        The identifier can be specified using several formats:

        - FunctionLogicalName
        - ModuleSerialNumber.FunctionIdentifier
        - ModuleSerialNumber.FunctionLogicalName
        - ModuleLogicalName.FunctionIdentifier
        - ModuleLogicalName.FunctionLogicalName


        This function does not require that the wireless LAN interface is online at the time
        it is invoked. The returned object is nevertheless valid.
        Use the method YWireless.isOnline() to test if the wireless LAN interface is
        indeed online at a given time. In case of ambiguity when looking for
        a wireless LAN interface by logical name, no error is notified: the first instance
        found is returned. The search is performed first by hardware name,
        then by logical name.

        @param yctx : a YAPI context
        @param func : a string that uniquely characterizes the wireless LAN interface, for instance
                YHUBWLN1.wireless.

        @return a YWireless object allowing you to drive the wireless LAN interface.
        """
        obj: Union[YWireless, None]
        obj = YFunction._FindFromCacheInContext(yctx, "Wireless", func)
        if obj is None:
            obj = YWireless(yctx, func)
            YFunction._AddToCache("Wireless", func, obj)
        return obj

    if not _IS_MICROPYTHON:
        async def registerValueCallback(self, callback: YWirelessValueCallback) -> int:
            """
            Registers the callback function that is invoked on every change of advertised value.
            The callback is invoked only during the execution of ySleep or yHandleEvents.
            This provides control over the time when the callback is triggered. For good responsiveness, remember to call
            one of these two functions periodically. To unregister a callback, pass a None pointer as argument.

            @param callback : the callback function to call, or a None pointer. The callback function should take two
                    arguments: the function object of which the value has changed, and the character string describing
                    the new advertised value.
            @noreturn
            """
            return await super().registerValueCallback(callback)

    async def startWlanScan(self) -> int:
        """
        Triggers a scan of the wireless frequency and builds the list of available networks.
        The scan forces a disconnection from the current network. At then end of the process, the
        the network interface attempts to reconnect to the previous network. During the scan, the wlanState
        switches to YWireless.WLANSTATE_DOWN, then to YWireless.WLANSTATE_SCANNING. When the scan is completed,
        get_wlanState() returns either YWireless.WLANSTATE_DOWN or YWireless.WLANSTATE_SCANNING. At this
        point, the list of detected network can be retrieved with the get_detectedWlans() method.

        On failure, throws an exception or returns a negative error code.
        """
        config: str
        config = await self.get_wlanConfig()
        # a full scan is triggered when a config is applied
        return await self.set_wlanConfig(config)

    async def joinNetwork(self, ssid: str, securityKey: str) -> int:
        """
        Changes the configuration of the wireless lan interface to connect to an existing
        access point (infrastructure mode).
        Remember to call the saveToFlash() method and then to reboot the module to apply this setting.

        @param ssid : the name of the network to connect to
        @param securityKey : the network key, as a character string

        @return YAPI.SUCCESS when the call succeeds.

        On failure, throws an exception or returns a negative error code.
        """
        return await self.set_wlanConfig("INFRA:%s\\%s" % (ssid, securityKey))

    async def adhocNetwork(self, ssid: str, securityKey: str) -> int:
        """
        Changes the configuration of the wireless lan interface to create an ad-hoc
        wireless network, without using an access point. On the YoctoHub-Wireless-g
        and YoctoHub-Wireless-n,
        you should use softAPNetwork() instead, which emulates an access point
        (Soft AP) which is more efficient and more widely supported than ad-hoc networks.

        When a security key is specified for an ad-hoc network, the network is protected
        by a WEP40 key (5 characters or 10 hexadecimal digits) or WEP128 key (13 characters
        or 26 hexadecimal digits). It is recommended to use a well-randomized WEP128 key
        using 26 hexadecimal digits to maximize security.
        Remember to call the saveToFlash() method and then to reboot the module
        to apply this setting.

        @param ssid : the name of the network to connect to
        @param securityKey : the network key, as a character string

        @return YAPI.SUCCESS when the call succeeds.

        On failure, throws an exception or returns a negative error code.
        """
        return await self.set_wlanConfig("ADHOC:%s\\%s" % (ssid, securityKey))

    async def softAPNetwork(self, ssid: str, securityKey: str) -> int:
        """
        Changes the configuration of the wireless lan interface to create a new wireless
        network by emulating a WiFi access point (Soft AP). This function can only be
        used with the YoctoHub-Wireless-g and the YoctoHub-Wireless-n.

        On the YoctoHub-Wireless-g, when a security key is specified for a SoftAP network,
        the network is protected by a WEP40 key (5 characters or 10 hexadecimal digits) or
        WEP128 key (13 characters or 26 hexadecimal digits). It is recommended to use a
        well-randomized WEP128 key using 26 hexadecimal digits to maximize security.

        On the YoctoHub-Wireless-n, when a security key is specified for a SoftAP network,
        the network will be protected by WPA2.

        Remember to call the saveToFlash() method and then to reboot the module to apply this setting.

        @param ssid : the name of the network to connect to
        @param securityKey : the network key, as a character string

        @return YAPI.SUCCESS when the call succeeds.

        On failure, throws an exception or returns a negative error code.
        """
        return await self.set_wlanConfig("SOFTAP:%s\\%s" % (ssid, securityKey))

    async def get_detectedWlans(self) -> list[YWlanRecord]:
        """
        Returns a list of YWlanRecord objects that describe detected Wireless networks.
        This list is not updated when the module is already connected to an access point (infrastructure mode).
        To force an update of this list, startWlanScan() must be called.
        Note that an languages without garbage collections, the returned list must be freed by the caller.

        @return a list of YWlanRecord objects, containing the SSID, channel,
                link quality and the type of security of the wireless network.

        On failure, throws an exception or returns an empty list.
        """
        json: xarray
        wlanlist: list[xarray] = []
        res: Union[list[YWlanRecord], None] = []

        json = await self._download("wlan.json?by=name")
        wlanlist = self._json_get_array(json)
        del res[:]
        for y in wlanlist:
            res.append(YWlanRecord(y.decode('latin-1')))
        return res

    # --- (end of generated code: YWireless implementation)



# -*- coding: utf-8 -*-
# ********************************************************************
#
#  $Id: svn_id $
#
#  Asyncio implementation of YThreshold
#
#  - - - - - - - - - License information: - - - - - - - - -
#
#  Copyright (C) 2011 and beyond by Yoctopuce Sarl, Switzerland.
#
#  Yoctopuce Sarl (hereafter Licensor) grants to you a perpetual
#  non-exclusive license to use, modify, copy and integrate this
#  file into your software for the sole purpose of interfacing
#  with Yoctopuce products.
#
#  You may reproduce and distribute copies of this file in
#  source or object form, as long as the sole purpose of this
#  code is to interface with Yoctopuce products. You must retain
#  this notice in the distributed source file.
#
#  You should refer to Yoctopuce General Terms and Conditions
#  for additional information regarding your rights and
#  obligations.
#
#  THE SOFTWARE AND DOCUMENTATION ARE PROVIDED 'AS IS' WITHOUT
#  WARRANTY OF ANY KIND, EITHER EXPRESS OR IMPLIED, INCLUDING
#  WITHOUT LIMITATION, ANY WARRANTY OF MERCHANTABILITY, FITNESS
#  FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO
#  EVENT SHALL LICENSOR BE LIABLE FOR ANY INCIDENTAL, SPECIAL,
#  INDIRECT OR CONSEQUENTIAL DAMAGES, LOST PROFITS OR LOST DATA,
#  COST OF PROCUREMENT OF SUBSTITUTE GOODS, TECHNOLOGY OR
#  SERVICES, ANY CLAIMS BY THIRD PARTIES (INCLUDING BUT NOT
#  LIMITED TO ANY DEFENSE THEREOF), ANY CLAIMS FOR INDEMNITY OR
#  CONTRIBUTION, OR OTHER SIMILAR COSTS, WHETHER ASSERTED ON THE
#  BASIS OF CONTRACT, TORT (INCLUDING NEGLIGENCE), BREACH OF
#  WARRANTY, OR OTHERWISE.
#
# *********************************************************************
"""
Yoctopuce library: Asyncio implementation of YThreshold
version: PATCH_WITH_VERSION
requires: yocto_api_aio
"""
from __future__ import annotations

import sys

# On MicroPython, code below will be wiped out at compile time
if sys.implementation.name != "micropython":
    # In CPython, enable edit-time type checking, including Final declaration
    from typing import Any, Union, Final
    from collections.abc import Callable, Awaitable
    from .yocto_api_aio import const, _IS_MICROPYTHON
else:
    # In our micropython VM, common generic types are global built-ins
    # Others such as TypeVar should be avoided when using micropython,
    # as they produce overhead in runtime code
    # Final is translated into const() expressions before compilation
    _IS_MICROPYTHON: Final[bool] = True # noqa

from .yocto_api_aio import (
    YAPIContext, YAPI, YAPI_Exception, HwId, hwid2str, YFunction
)

# --- (YThreshold class start)
if not _IS_MICROPYTHON:
    # For CPython, use strongly typed callback types
    try:
        YThresholdValueCallback = Union[Callable[['YThreshold', str], Awaitable[None]], None]
    except TypeError:
        YThresholdValueCallback = Union[Callable, Awaitable]

# noinspection PyProtectedMember
class YThreshold(YFunction):
    """
    The Threshold class allows you define a threshold on a Yoctopuce sensor
    to trigger a predefined action, on specific devices where this is implemented.

    """
    # --- (end of YThreshold class start)
    if not _IS_MICROPYTHON:
        # --- (YThreshold return codes)
        TARGETSENSOR_INVALID: Final[str] = YAPI.INVALID_STRING
        ALERTLEVEL_INVALID: Final[float] = YAPI.INVALID_DOUBLE
        SAFELEVEL_INVALID: Final[float] = YAPI.INVALID_DOUBLE
        THRESHOLDSTATE_SAFE: Final[int] = 0
        THRESHOLDSTATE_ALERT: Final[int] = 1
        THRESHOLDSTATE_INVALID: Final[int] = -1
        # --- (end of YThreshold return codes)

    # --- (YThreshold attributes declaration)
    _thresholdState: int
    _targetSensor: str
    _alertLevel: float
    _safeLevel: float
    _valueCallback: YThresholdValueCallback
    # --- (end of YThreshold attributes declaration)


    def __init__(self, yctx: YAPIContext, func: str):
        super().__init__(yctx, func)
        self._className = 'Threshold'
        # --- (YThreshold constructor)
        self._thresholdState = YThreshold.THRESHOLDSTATE_INVALID
        self._targetSensor = YThreshold.TARGETSENSOR_INVALID
        self._alertLevel = YThreshold.ALERTLEVEL_INVALID
        self._safeLevel = YThreshold.SAFELEVEL_INVALID
        # --- (end of YThreshold constructor)

    # --- (YThreshold implementation)

    @staticmethod
    def FirstThreshold() -> Union[YThreshold, None]:
        """
        Starts the enumeration of threshold functions currently accessible.
        Use the method YThreshold.nextThreshold() to iterate on
        next threshold functions.

        @return a pointer to a YThreshold object, corresponding to
                the first threshold function currently online, or a None pointer
                if there are none.
        """
        next_hwid: Union[HwId, None] = YAPI._yHash.getFirstHardwareId('Threshold')
        if not next_hwid:
            return None
        return YThreshold.FindThreshold(hwid2str(next_hwid))

    @staticmethod
    def FirstThresholdInContext(yctx: YAPIContext) -> Union[YThreshold, None]:
        """
        Starts the enumeration of threshold functions currently accessible.
        Use the method YThreshold.nextThreshold() to iterate on
        next threshold functions.

        @param yctx : a YAPI context.

        @return a pointer to a YThreshold object, corresponding to
                the first threshold function currently online, or a None pointer
                if there are none.
        """
        next_hwid: Union[HwId, None] = yctx._yHash.getFirstHardwareId('Threshold')
        if not next_hwid:
            return None
        return YThreshold.FindThresholdInContext(yctx, hwid2str(next_hwid))

    def nextThreshold(self):
        """
        Continues the enumeration of threshold functions started using yFirstThreshold().
        Caution: You can't make any assumption about the returned threshold functions order.
        If you want to find a specific a threshold function, use Threshold.findThreshold()
        and a hardwareID or a logical name.

        @return a pointer to a YThreshold object, corresponding to
                a threshold function currently online, or a None pointer
                if there are no more threshold functions to enumerate.
        """
        next_hwid: Union[HwId, None] = None
        try:
            hwid: HwId = self._yapi._yHash.resolveHwID(self._className, self._func)
            next_hwid = self._yapi._yHash.getNextHardwareId(self._className, hwid)
        except YAPI_Exception:
            pass
        if not next_hwid:
            return None
        return YThreshold.FindThresholdInContext(self._yapi, hwid2str(next_hwid))

    def _parseAttr(self, json_val: dict) -> None:
        if 'thresholdState' in json_val:
            self._thresholdState = json_val["thresholdState"]
        if 'targetSensor' in json_val:
            self._targetSensor = json_val["targetSensor"]
        if 'alertLevel' in json_val:
            self._alertLevel = round(json_val["alertLevel"] / 65.536) / 1000.0
        if 'safeLevel' in json_val:
            self._safeLevel = round(json_val["safeLevel"] / 65.536) / 1000.0
        super()._parseAttr(json_val)

    async def get_thresholdState(self) -> int:
        """
        Returns current state of the threshold function.

        @return either YThreshold.THRESHOLDSTATE_SAFE or YThreshold.THRESHOLDSTATE_ALERT, according to
        current state of the threshold function

        On failure, throws an exception or returns YThreshold.THRESHOLDSTATE_INVALID.
        """
        res: int
        if self._cacheExpiration <= YAPI.GetTickCount():
            if await self.load(self._yapi.GetCacheValidity()) != YAPI.SUCCESS:
                return YThreshold.THRESHOLDSTATE_INVALID
        res = self._thresholdState
        return res

    async def get_targetSensor(self) -> str:
        """
        Returns the name of the sensor monitored by the threshold function.

        @return a string corresponding to the name of the sensor monitored by the threshold function

        On failure, throws an exception or returns YThreshold.TARGETSENSOR_INVALID.
        """
        res: str
        if self._cacheExpiration <= YAPI.GetTickCount():
            if await self.load(self._yapi.GetCacheValidity()) != YAPI.SUCCESS:
                return YThreshold.TARGETSENSOR_INVALID
        res = self._targetSensor
        return res

    async def set_alertLevel(self, newval: float) -> int:
        """
        Changes the sensor alert level triggering the threshold function.
        Remember to call the matching module saveToFlash()
        method if you want to preserve the setting after reboot.

        @param newval : a floating point number corresponding to the sensor alert level triggering the
        threshold function

        @return YAPI.SUCCESS if the call succeeds.

        On failure, throws an exception or returns a negative error code.
        """
        rest_val = str(int(round(newval * 65536.0, 1)))
        return await self._setAttr("alertLevel", rest_val)

    async def get_alertLevel(self) -> float:
        """
        Returns the sensor alert level, triggering the threshold function.

        @return a floating point number corresponding to the sensor alert level, triggering the threshold function

        On failure, throws an exception or returns YThreshold.ALERTLEVEL_INVALID.
        """
        res: float
        if self._cacheExpiration <= YAPI.GetTickCount():
            if await self.load(self._yapi.GetCacheValidity()) != YAPI.SUCCESS:
                return YThreshold.ALERTLEVEL_INVALID
        res = self._alertLevel
        return res

    async def set_safeLevel(self, newval: float) -> int:
        """
        Changes the sensor acceptable level for disabling the threshold function.
        Remember to call the matching module saveToFlash()
        method if you want to preserve the setting after reboot.

        @param newval : a floating point number corresponding to the sensor acceptable level for disabling
        the threshold function

        @return YAPI.SUCCESS if the call succeeds.

        On failure, throws an exception or returns a negative error code.
        """
        rest_val = str(int(round(newval * 65536.0, 1)))
        return await self._setAttr("safeLevel", rest_val)

    async def get_safeLevel(self) -> float:
        """
        Returns the sensor acceptable level for disabling the threshold function.

        @return a floating point number corresponding to the sensor acceptable level for disabling the
        threshold function

        On failure, throws an exception or returns YThreshold.SAFELEVEL_INVALID.
        """
        res: float
        if self._cacheExpiration <= YAPI.GetTickCount():
            if await self.load(self._yapi.GetCacheValidity()) != YAPI.SUCCESS:
                return YThreshold.SAFELEVEL_INVALID
        res = self._safeLevel
        return res

    @staticmethod
    def FindThreshold(func: str) -> YThreshold:
        """
        Retrieves a threshold function for a given identifier.
        The identifier can be specified using several formats:

        - FunctionLogicalName
        - ModuleSerialNumber.FunctionIdentifier
        - ModuleSerialNumber.FunctionLogicalName
        - ModuleLogicalName.FunctionIdentifier
        - ModuleLogicalName.FunctionLogicalName


        This function does not require that the threshold function is online at the time
        it is invoked. The returned object is nevertheless valid.
        Use the method YThreshold.isOnline() to test if the threshold function is
        indeed online at a given time. In case of ambiguity when looking for
        a threshold function by logical name, no error is notified: the first instance
        found is returned. The search is performed first by hardware name,
        then by logical name.

        If a call to this object's is_online() method returns FALSE although
        you are certain that the matching device is plugged, make sure that you did
        call registerHub() at application initialization time.

        @param func : a string that uniquely characterizes the threshold function, for instance
                MyDevice.threshold1.

        @return a YThreshold object allowing you to drive the threshold function.
        """
        obj: Union[YThreshold, None]
        obj = YFunction._FindFromCache("Threshold", func)
        if obj is None:
            obj = YThreshold(YAPI, func)
            YFunction._AddToCache("Threshold", func, obj)
        return obj

    @staticmethod
    def FindThresholdInContext(yctx: YAPIContext, func: str) -> YThreshold:
        """
        Retrieves a threshold function for a given identifier in a YAPI context.
        The identifier can be specified using several formats:

        - FunctionLogicalName
        - ModuleSerialNumber.FunctionIdentifier
        - ModuleSerialNumber.FunctionLogicalName
        - ModuleLogicalName.FunctionIdentifier
        - ModuleLogicalName.FunctionLogicalName


        This function does not require that the threshold function is online at the time
        it is invoked. The returned object is nevertheless valid.
        Use the method YThreshold.isOnline() to test if the threshold function is
        indeed online at a given time. In case of ambiguity when looking for
        a threshold function by logical name, no error is notified: the first instance
        found is returned. The search is performed first by hardware name,
        then by logical name.

        @param yctx : a YAPI context
        @param func : a string that uniquely characterizes the threshold function, for instance
                MyDevice.threshold1.

        @return a YThreshold object allowing you to drive the threshold function.
        """
        obj: Union[YThreshold, None]
        obj = YFunction._FindFromCacheInContext(yctx, "Threshold", func)
        if obj is None:
            obj = YThreshold(yctx, func)
            YFunction._AddToCache("Threshold", func, obj)
        return obj

    if not _IS_MICROPYTHON:
        async def registerValueCallback(self, callback: YThresholdValueCallback) -> int:
            """
            Registers the callback function that is invoked on every change of advertised value.
            The callback is invoked only during the execution of ySleep or yHandleEvents.
            This provides control over the time when the callback is triggered. For good responsiveness, remember to call
            one of these two functions periodically. To unregister a callback, pass a None pointer as argument.

            @param callback : the callback function to call, or a None pointer. The callback function should take two
                    arguments: the function object of which the value has changed, and the character string describing
                    the new advertised value.
            @noreturn
            """
            return await super().registerValueCallback(callback)

    # --- (end of YThreshold implementation)




# -*- coding: utf-8 -*-
# ********************************************************************
#
#  $Id: svn_id $
#
#  Asyncio implementation of YWakeUpSchedule
#
#  - - - - - - - - - License information: - - - - - - - - -
#
#  Copyright (C) 2011 and beyond by Yoctopuce Sarl, Switzerland.
#
#  Yoctopuce Sarl (hereafter Licensor) grants to you a perpetual
#  non-exclusive license to use, modify, copy and integrate this
#  file into your software for the sole purpose of interfacing
#  with Yoctopuce products.
#
#  You may reproduce and distribute copies of this file in
#  source or object form, as long as the sole purpose of this
#  code is to interface with Yoctopuce products. You must retain
#  this notice in the distributed source file.
#
#  You should refer to Yoctopuce General Terms and Conditions
#  for additional information regarding your rights and
#  obligations.
#
#  THE SOFTWARE AND DOCUMENTATION ARE PROVIDED 'AS IS' WITHOUT
#  WARRANTY OF ANY KIND, EITHER EXPRESS OR IMPLIED, INCLUDING
#  WITHOUT LIMITATION, ANY WARRANTY OF MERCHANTABILITY, FITNESS
#  FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO
#  EVENT SHALL LICENSOR BE LIABLE FOR ANY INCIDENTAL, SPECIAL,
#  INDIRECT OR CONSEQUENTIAL DAMAGES, LOST PROFITS OR LOST DATA,
#  COST OF PROCUREMENT OF SUBSTITUTE GOODS, TECHNOLOGY OR
#  SERVICES, ANY CLAIMS BY THIRD PARTIES (INCLUDING BUT NOT
#  LIMITED TO ANY DEFENSE THEREOF), ANY CLAIMS FOR INDEMNITY OR
#  CONTRIBUTION, OR OTHER SIMILAR COSTS, WHETHER ASSERTED ON THE
#  BASIS OF CONTRACT, TORT (INCLUDING NEGLIGENCE), BREACH OF
#  WARRANTY, OR OTHERWISE.
#
# *********************************************************************
"""
Yoctopuce library: Asyncio implementation of YWakeUpSchedule
version: PATCH_WITH_VERSION
requires: yocto_api_aio
"""
from __future__ import annotations

import sys

# On MicroPython, code below will be wiped out at compile time
if sys.implementation.name != "micropython":
    # In CPython, enable edit-time type checking, including Final declaration
    from typing import Any, Union, Final
    from collections.abc import Callable, Awaitable
    from .yocto_api_aio import const, _IS_MICROPYTHON
else:
    # In our micropython VM, common generic types are global built-ins
    # Others such as TypeVar should be avoided when using micropython,
    # as they produce overhead in runtime code
    # Final is translated into const() expressions before compilation
    _IS_MICROPYTHON: Final[bool] = True # noqa

from .yocto_api_aio import (
    YAPIContext, YAPI, YAPI_Exception, HwId, hwid2str, YFunction
)

# --- (YWakeUpSchedule class start)
if not _IS_MICROPYTHON:
    # For CPython, use strongly typed callback types
    try:
        YWakeUpScheduleValueCallback = Union[Callable[['YWakeUpSchedule', str], Awaitable[None]], None]
    except TypeError:
        YWakeUpScheduleValueCallback = Union[Callable, Awaitable]

# noinspection PyProtectedMember
class YWakeUpSchedule(YFunction):
    """
    The YWakeUpSchedule class implements a wake up condition. The wake up time is
    specified as a set of months and/or days and/or hours and/or minutes when the
    wake up should happen.

    """
    # --- (end of YWakeUpSchedule class start)
    if not _IS_MICROPYTHON:
        # --- (YWakeUpSchedule return codes)
        MINUTESA_INVALID: Final[int] = YAPI.INVALID_UINT
        MINUTESB_INVALID: Final[int] = YAPI.INVALID_UINT
        HOURS_INVALID: Final[int] = YAPI.INVALID_UINT
        WEEKDAYS_INVALID: Final[int] = YAPI.INVALID_UINT
        MONTHDAYS_INVALID: Final[int] = YAPI.INVALID_UINT
        MONTHS_INVALID: Final[int] = YAPI.INVALID_UINT
        SECONDSBEFORE_INVALID: Final[int] = YAPI.INVALID_UINT
        NEXTOCCURENCE_INVALID: Final[int] = YAPI.INVALID_LONG
        # --- (end of YWakeUpSchedule return codes)

    # --- (YWakeUpSchedule attributes declaration)
    _minutesA: int
    _minutesB: int
    _hours: int
    _weekDays: int
    _monthDays: int
    _months: int
    _secondsBefore: int
    _nextOccurence: int
    _valueCallback: YWakeUpScheduleValueCallback
    # --- (end of YWakeUpSchedule attributes declaration)


    def __init__(self, yctx: YAPIContext, func: str):
        super().__init__(yctx, func)
        self._className = 'WakeUpSchedule'
        # --- (YWakeUpSchedule constructor)
        self._minutesA = YWakeUpSchedule.MINUTESA_INVALID
        self._minutesB = YWakeUpSchedule.MINUTESB_INVALID
        self._hours = YWakeUpSchedule.HOURS_INVALID
        self._weekDays = YWakeUpSchedule.WEEKDAYS_INVALID
        self._monthDays = YWakeUpSchedule.MONTHDAYS_INVALID
        self._months = YWakeUpSchedule.MONTHS_INVALID
        self._secondsBefore = YWakeUpSchedule.SECONDSBEFORE_INVALID
        self._nextOccurence = YWakeUpSchedule.NEXTOCCURENCE_INVALID
        # --- (end of YWakeUpSchedule constructor)

    # --- (YWakeUpSchedule implementation)

    @staticmethod
    def FirstWakeUpSchedule() -> Union[YWakeUpSchedule, None]:
        """
        Starts the enumeration of wake up schedules currently accessible.
        Use the method YWakeUpSchedule.nextWakeUpSchedule() to iterate on
        next wake up schedules.

        @return a pointer to a YWakeUpSchedule object, corresponding to
                the first wake up schedule currently online, or a None pointer
                if there are none.
        """
        next_hwid: Union[HwId, None] = YAPI._yHash.getFirstHardwareId('WakeUpSchedule')
        if not next_hwid:
            return None
        return YWakeUpSchedule.FindWakeUpSchedule(hwid2str(next_hwid))

    @staticmethod
    def FirstWakeUpScheduleInContext(yctx: YAPIContext) -> Union[YWakeUpSchedule, None]:
        """
        Starts the enumeration of wake up schedules currently accessible.
        Use the method YWakeUpSchedule.nextWakeUpSchedule() to iterate on
        next wake up schedules.

        @param yctx : a YAPI context.

        @return a pointer to a YWakeUpSchedule object, corresponding to
                the first wake up schedule currently online, or a None pointer
                if there are none.
        """
        next_hwid: Union[HwId, None] = yctx._yHash.getFirstHardwareId('WakeUpSchedule')
        if not next_hwid:
            return None
        return YWakeUpSchedule.FindWakeUpScheduleInContext(yctx, hwid2str(next_hwid))

    def nextWakeUpSchedule(self):
        """
        Continues the enumeration of wake up schedules started using yFirstWakeUpSchedule().
        Caution: You can't make any assumption about the returned wake up schedules order.
        If you want to find a specific a wake up schedule, use WakeUpSchedule.findWakeUpSchedule()
        and a hardwareID or a logical name.

        @return a pointer to a YWakeUpSchedule object, corresponding to
                a wake up schedule currently online, or a None pointer
                if there are no more wake up schedules to enumerate.
        """
        next_hwid: Union[HwId, None] = None
        try:
            hwid: HwId = self._yapi._yHash.resolveHwID(self._className, self._func)
            next_hwid = self._yapi._yHash.getNextHardwareId(self._className, hwid)
        except YAPI_Exception:
            pass
        if not next_hwid:
            return None
        return YWakeUpSchedule.FindWakeUpScheduleInContext(self._yapi, hwid2str(next_hwid))

    def _parseAttr(self, json_val: dict) -> None:
        if 'minutesA' in json_val:
            self._minutesA = json_val["minutesA"]
        if 'minutesB' in json_val:
            self._minutesB = json_val["minutesB"]
        if 'hours' in json_val:
            self._hours = json_val["hours"]
        if 'weekDays' in json_val:
            self._weekDays = json_val["weekDays"]
        if 'monthDays' in json_val:
            self._monthDays = json_val["monthDays"]
        if 'months' in json_val:
            self._months = json_val["months"]
        if 'secondsBefore' in json_val:
            self._secondsBefore = json_val["secondsBefore"]
        if 'nextOccurence' in json_val:
            self._nextOccurence = json_val["nextOccurence"]
        super()._parseAttr(json_val)

    async def get_minutesA(self) -> int:
        """
        Returns the minutes in the 00-29 interval of each hour scheduled for wake up.

        @return an integer corresponding to the minutes in the 00-29 interval of each hour scheduled for wake up

        On failure, throws an exception or returns YWakeUpSchedule.MINUTESA_INVALID.
        """
        res: int
        if self._cacheExpiration <= YAPI.GetTickCount():
            if await self.load(self._yapi.GetCacheValidity()) != YAPI.SUCCESS:
                return YWakeUpSchedule.MINUTESA_INVALID
        res = self._minutesA
        return res

    async def set_minutesA(self, newval: int) -> int:
        """
        Changes the minutes in the 00-29 interval when a wake up must take place.
        Remember to call the saveToFlash() method of the module if the
        modification must be kept.

        @param newval : an integer corresponding to the minutes in the 00-29 interval when a wake up must take place

        @return YAPI.SUCCESS if the call succeeds.

        On failure, throws an exception or returns a negative error code.
        """
        rest_val = str(newval)
        return await self._setAttr("minutesA", rest_val)

    async def get_minutesB(self) -> int:
        """
        Returns the minutes in the 30-59 interval of each hour scheduled for wake up.

        @return an integer corresponding to the minutes in the 30-59 interval of each hour scheduled for wake up

        On failure, throws an exception or returns YWakeUpSchedule.MINUTESB_INVALID.
        """
        res: int
        if self._cacheExpiration <= YAPI.GetTickCount():
            if await self.load(self._yapi.GetCacheValidity()) != YAPI.SUCCESS:
                return YWakeUpSchedule.MINUTESB_INVALID
        res = self._minutesB
        return res

    async def set_minutesB(self, newval: int) -> int:
        """
        Changes the minutes in the 30-59 interval when a wake up must take place.
        Remember to call the saveToFlash() method of the module if the
        modification must be kept.

        @param newval : an integer corresponding to the minutes in the 30-59 interval when a wake up must take place

        @return YAPI.SUCCESS if the call succeeds.

        On failure, throws an exception or returns a negative error code.
        """
        rest_val = str(newval)
        return await self._setAttr("minutesB", rest_val)

    async def get_hours(self) -> int:
        """
        Returns the hours scheduled for wake up.

        @return an integer corresponding to the hours scheduled for wake up

        On failure, throws an exception or returns YWakeUpSchedule.HOURS_INVALID.
        """
        res: int
        if self._cacheExpiration <= YAPI.GetTickCount():
            if await self.load(self._yapi.GetCacheValidity()) != YAPI.SUCCESS:
                return YWakeUpSchedule.HOURS_INVALID
        res = self._hours
        return res

    async def set_hours(self, newval: int) -> int:
        """
        Changes the hours when a wake up must take place.
        Remember to call the saveToFlash() method of the module if the
        modification must be kept.

        @param newval : an integer corresponding to the hours when a wake up must take place

        @return YAPI.SUCCESS if the call succeeds.

        On failure, throws an exception or returns a negative error code.
        """
        rest_val = str(newval)
        return await self._setAttr("hours", rest_val)

    async def get_weekDays(self) -> int:
        """
        Returns the days of the week scheduled for wake up.

        @return an integer corresponding to the days of the week scheduled for wake up

        On failure, throws an exception or returns YWakeUpSchedule.WEEKDAYS_INVALID.
        """
        res: int
        if self._cacheExpiration <= YAPI.GetTickCount():
            if await self.load(self._yapi.GetCacheValidity()) != YAPI.SUCCESS:
                return YWakeUpSchedule.WEEKDAYS_INVALID
        res = self._weekDays
        return res

    async def set_weekDays(self, newval: int) -> int:
        """
        Changes the days of the week when a wake up must take place.
        Remember to call the saveToFlash() method of the module if the
        modification must be kept.

        @param newval : an integer corresponding to the days of the week when a wake up must take place

        @return YAPI.SUCCESS if the call succeeds.

        On failure, throws an exception or returns a negative error code.
        """
        rest_val = str(newval)
        return await self._setAttr("weekDays", rest_val)

    async def get_monthDays(self) -> int:
        """
        Returns the days of the month scheduled for wake up.

        @return an integer corresponding to the days of the month scheduled for wake up

        On failure, throws an exception or returns YWakeUpSchedule.MONTHDAYS_INVALID.
        """
        res: int
        if self._cacheExpiration <= YAPI.GetTickCount():
            if await self.load(self._yapi.GetCacheValidity()) != YAPI.SUCCESS:
                return YWakeUpSchedule.MONTHDAYS_INVALID
        res = self._monthDays
        return res

    async def set_monthDays(self, newval: int) -> int:
        """
        Changes the days of the month when a wake up must take place.
        Remember to call the saveToFlash() method of the module if the
        modification must be kept.

        @param newval : an integer corresponding to the days of the month when a wake up must take place

        @return YAPI.SUCCESS if the call succeeds.

        On failure, throws an exception or returns a negative error code.
        """
        rest_val = str(newval)
        return await self._setAttr("monthDays", rest_val)

    async def get_months(self) -> int:
        """
        Returns the months scheduled for wake up.

        @return an integer corresponding to the months scheduled for wake up

        On failure, throws an exception or returns YWakeUpSchedule.MONTHS_INVALID.
        """
        res: int
        if self._cacheExpiration <= YAPI.GetTickCount():
            if await self.load(self._yapi.GetCacheValidity()) != YAPI.SUCCESS:
                return YWakeUpSchedule.MONTHS_INVALID
        res = self._months
        return res

    async def set_months(self, newval: int) -> int:
        """
        Changes the months when a wake up must take place.
        Remember to call the saveToFlash() method of the module if the
        modification must be kept.

        @param newval : an integer corresponding to the months when a wake up must take place

        @return YAPI.SUCCESS if the call succeeds.

        On failure, throws an exception or returns a negative error code.
        """
        rest_val = str(newval)
        return await self._setAttr("months", rest_val)

    async def get_secondsBefore(self) -> int:
        """
        Returns the number of seconds to anticipate wake-up time to allow
        the system to power-up.

        @return an integer corresponding to the number of seconds to anticipate wake-up time to allow
                the system to power-up

        On failure, throws an exception or returns YWakeUpSchedule.SECONDSBEFORE_INVALID.
        """
        res: int
        if self._cacheExpiration <= YAPI.GetTickCount():
            if await self.load(self._yapi.GetCacheValidity()) != YAPI.SUCCESS:
                return YWakeUpSchedule.SECONDSBEFORE_INVALID
        res = self._secondsBefore
        return res

    async def set_secondsBefore(self, newval: int) -> int:
        """
        Changes the number of seconds to anticipate wake-up time to allow
        the system to power-up.
        Remember to call the saveToFlash() method of the module if the
        modification must be kept.

        @param newval : an integer corresponding to the number of seconds to anticipate wake-up time to allow
                the system to power-up

        @return YAPI.SUCCESS if the call succeeds.

        On failure, throws an exception or returns a negative error code.
        """
        rest_val = str(newval)
        return await self._setAttr("secondsBefore", rest_val)

    async def get_nextOccurence(self) -> int:
        """
        Returns the date/time (seconds) of the next wake up occurrence.

        @return an integer corresponding to the date/time (seconds) of the next wake up occurrence

        On failure, throws an exception or returns YWakeUpSchedule.NEXTOCCURENCE_INVALID.
        """
        res: int
        if self._cacheExpiration <= YAPI.GetTickCount():
            if await self.load(self._yapi.GetCacheValidity()) != YAPI.SUCCESS:
                return YWakeUpSchedule.NEXTOCCURENCE_INVALID
        res = self._nextOccurence
        return res

    @staticmethod
    def FindWakeUpSchedule(func: str) -> YWakeUpSchedule:
        """
        Retrieves a wake up schedule for a given identifier.
        The identifier can be specified using several formats:

        - FunctionLogicalName
        - ModuleSerialNumber.FunctionIdentifier
        - ModuleSerialNumber.FunctionLogicalName
        - ModuleLogicalName.FunctionIdentifier
        - ModuleLogicalName.FunctionLogicalName


        This function does not require that the wake up schedule is online at the time
        it is invoked. The returned object is nevertheless valid.
        Use the method YWakeUpSchedule.isOnline() to test if the wake up schedule is
        indeed online at a given time. In case of ambiguity when looking for
        a wake up schedule by logical name, no error is notified: the first instance
        found is returned. The search is performed first by hardware name,
        then by logical name.

        If a call to this object's is_online() method returns FALSE although
        you are certain that the matching device is plugged, make sure that you did
        call registerHub() at application initialization time.

        @param func : a string that uniquely characterizes the wake up schedule, for instance
                YHUBGSM5.wakeUpSchedule1.

        @return a YWakeUpSchedule object allowing you to drive the wake up schedule.
        """
        obj: Union[YWakeUpSchedule, None]
        obj = YFunction._FindFromCache("WakeUpSchedule", func)
        if obj is None:
            obj = YWakeUpSchedule(YAPI, func)
            YFunction._AddToCache("WakeUpSchedule", func, obj)
        return obj

    @staticmethod
    def FindWakeUpScheduleInContext(yctx: YAPIContext, func: str) -> YWakeUpSchedule:
        """
        Retrieves a wake up schedule for a given identifier in a YAPI context.
        The identifier can be specified using several formats:

        - FunctionLogicalName
        - ModuleSerialNumber.FunctionIdentifier
        - ModuleSerialNumber.FunctionLogicalName
        - ModuleLogicalName.FunctionIdentifier
        - ModuleLogicalName.FunctionLogicalName


        This function does not require that the wake up schedule is online at the time
        it is invoked. The returned object is nevertheless valid.
        Use the method YWakeUpSchedule.isOnline() to test if the wake up schedule is
        indeed online at a given time. In case of ambiguity when looking for
        a wake up schedule by logical name, no error is notified: the first instance
        found is returned. The search is performed first by hardware name,
        then by logical name.

        @param yctx : a YAPI context
        @param func : a string that uniquely characterizes the wake up schedule, for instance
                YHUBGSM5.wakeUpSchedule1.

        @return a YWakeUpSchedule object allowing you to drive the wake up schedule.
        """
        obj: Union[YWakeUpSchedule, None]
        obj = YFunction._FindFromCacheInContext(yctx, "WakeUpSchedule", func)
        if obj is None:
            obj = YWakeUpSchedule(yctx, func)
            YFunction._AddToCache("WakeUpSchedule", func, obj)
        return obj

    if not _IS_MICROPYTHON:
        async def registerValueCallback(self, callback: YWakeUpScheduleValueCallback) -> int:
            """
            Registers the callback function that is invoked on every change of advertised value.
            The callback is invoked only during the execution of ySleep or yHandleEvents.
            This provides control over the time when the callback is triggered. For good responsiveness, remember to call
            one of these two functions periodically. To unregister a callback, pass a None pointer as argument.

            @param callback : the callback function to call, or a None pointer. The callback function should take two
                    arguments: the function object of which the value has changed, and the character string describing
                    the new advertised value.
            @noreturn
            """
            return await super().registerValueCallback(callback)

    async def get_minutes(self) -> int:
        """
        Returns all the minutes of each hour that are scheduled for wake up.
        """
        res: int

        res = await self.get_minutesB()
        res = (res << 30)
        res = res + await self.get_minutesA()
        return res

    async def set_minutes(self, bitmap: int) -> int:
        """
        Changes all the minutes where a wake up must take place.

        @param bitmap : Minutes 00-59 of each hour scheduled for wake up.

        @return YAPI.SUCCESS if the call succeeds.

        On failure, throws an exception or returns a negative error code.
        """
        await self.set_minutesA((bitmap & 0x3fffffff))
        bitmap = (bitmap >> 30)
        return await self.set_minutesB((bitmap & 0x3fffffff))

    # --- (end of YWakeUpSchedule implementation)




# -*- coding: utf-8 -*-
# ********************************************************************
#
#  $Id: svn_id $
#
#  High-level API for YNetwork
#
#  - - - - - - - - - License information: - - - - - - - - -
#
#  Copyright (C) 2011 and beyond by Yoctopuce Sarl, Switzerland.
#
#  Yoctopuce Sarl (hereafter Licensor) grants to you a perpetual
#  non-exclusive license to use, modify, copy and integrate this
#  file into your software for the sole purpose of interfacing
#  with Yoctopuce products.
#
#  You may reproduce and distribute copies of this file in
#  source or object form, as long as the sole purpose of this
#  code is to interface with Yoctopuce products. You must retain
#  this notice in the distributed source file.
#
#  You should refer to Yoctopuce General Terms and Conditions
#  for additional information regarding your rights and
#  obligations.
#
#  THE SOFTWARE AND DOCUMENTATION ARE PROVIDED 'AS IS' WITHOUT
#  WARRANTY OF ANY KIND, EITHER EXPRESS OR IMPLIED, INCLUDING
#  WITHOUT LIMITATION, ANY WARRANTY OF MERCHANTABILITY, FITNESS
#  FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO
#  EVENT SHALL LICENSOR BE LIABLE FOR ANY INCIDENTAL, SPECIAL,
#  INDIRECT OR CONSEQUENTIAL DAMAGES, LOST PROFITS OR LOST DATA,
#  COST OF PROCUREMENT OF SUBSTITUTE GOODS, TECHNOLOGY OR
#  SERVICES, ANY CLAIMS BY THIRD PARTIES (INCLUDING BUT NOT
#  LIMITED TO ANY DEFENSE THEREOF), ANY CLAIMS FOR INDEMNITY OR
#  CONTRIBUTION, OR OTHER SIMILAR COSTS, WHETHER ASSERTED ON THE
#  BASIS OF CONTRACT, TORT (INCLUDING NEGLIGENCE), BREACH OF
#  WARRANTY, OR OTHERWISE.
#
# *********************************************************************
"""
Yoctopuce library: High-level API for YNetwork
version: PATCH_WITH_VERSION
requires: yocto_network_aio
requires: yocto_api
"""
from __future__ import annotations

import sys

# On MicroPython, code below will be wiped out at compile time
if sys.implementation.name != "micropython":
    # In CPython, enable edit-time type checking, including Final declaration
    from typing import Union, Final
    from collections.abc import Callable, Awaitable
    from .yocto_api import _IS_MICROPYTHON, _DYNAMIC_HELPERS
else:
    # In our micropython VM, common generic types are global built-ins
    # Others such as TypeVar should be avoided when using micropython,
    # as they produce overhead in runtime code
    # Final is translated into const() expressions before compilation
    _IS_MICROPYTHON: Final[bool] = True # noqa
    _DYNAMIC_HELPERS: Final[bool] = True # noqa

from .yocto_network_aio import YNetwork as YNetwork_aio
from .yocto_api import (
    YAPIContext, YAPI, YFunction
)

# --- (YNetwork class start)
if not _IS_MICROPYTHON:
    # For CPython, use strongly typed callback types
    try:
        YNetworkValueCallback = Union[Callable[['YNetwork', str], Awaitable[None]], None]
    except TypeError:
        YNetworkValueCallback = Union[Callable, Awaitable]

# noinspection PyProtectedMember
class YNetwork(YFunction):
    """
    YNetwork objects provide access to TCP/IP parameters of Yoctopuce
    devices that include a built-in network interface.

    """
    _aio: YNetwork_aio
    # --- (end of YNetwork class start)
    if not _IS_MICROPYTHON:
        # --- (YNetwork return codes)
        MACADDRESS_INVALID: Final[str] = YAPI.INVALID_STRING
        IPADDRESS_INVALID: Final[str] = YAPI.INVALID_STRING
        SUBNETMASK_INVALID: Final[str] = YAPI.INVALID_STRING
        ROUTER_INVALID: Final[str] = YAPI.INVALID_STRING
        CURRENTDNS_INVALID: Final[str] = YAPI.INVALID_STRING
        IPCONFIG_INVALID: Final[str] = YAPI.INVALID_STRING
        PRIMARYDNS_INVALID: Final[str] = YAPI.INVALID_STRING
        SECONDARYDNS_INVALID: Final[str] = YAPI.INVALID_STRING
        NTPSERVER_INVALID: Final[str] = YAPI.INVALID_STRING
        USERPASSWORD_INVALID: Final[str] = YAPI.INVALID_STRING
        ADMINPASSWORD_INVALID: Final[str] = YAPI.INVALID_STRING
        HTTPPORT_INVALID: Final[int] = YAPI.INVALID_UINT
        HTTPSPORT_INVALID: Final[int] = YAPI.INVALID_UINT
        DEFAULTPAGE_INVALID: Final[str] = YAPI.INVALID_STRING
        WWWWATCHDOGDELAY_INVALID: Final[int] = YAPI.INVALID_UINT
        CALLBACKURL_INVALID: Final[str] = YAPI.INVALID_STRING
        CALLBACKCREDENTIALS_INVALID: Final[str] = YAPI.INVALID_STRING
        CALLBACKINITIALDELAY_INVALID: Final[int] = YAPI.INVALID_UINT
        CALLBACKSCHEDULE_INVALID: Final[str] = YAPI.INVALID_STRING
        CALLBACKMINDELAY_INVALID: Final[int] = YAPI.INVALID_UINT
        CALLBACKMAXDELAY_INVALID: Final[int] = YAPI.INVALID_UINT
        POECURRENT_INVALID: Final[int] = YAPI.INVALID_UINT
        READINESS_DOWN: Final[int] = 0
        READINESS_EXISTS: Final[int] = 1
        READINESS_LINKED: Final[int] = 2
        READINESS_LAN_OK: Final[int] = 3
        READINESS_WWW_OK: Final[int] = 4
        READINESS_INVALID: Final[int] = -1
        SECURITYMODE_UNDEFINED: Final[int] = 0
        SECURITYMODE_LEGACY: Final[int] = 1
        SECURITYMODE_MIXED: Final[int] = 2
        SECURITYMODE_SECURE: Final[int] = 3
        SECURITYMODE_INVALID: Final[int] = -1
        DISCOVERABLE_FALSE: Final[int] = 0
        DISCOVERABLE_TRUE: Final[int] = 1
        DISCOVERABLE_INVALID: Final[int] = -1
        CALLBACKMETHOD_POST: Final[int] = 0
        CALLBACKMETHOD_GET: Final[int] = 1
        CALLBACKMETHOD_PUT: Final[int] = 2
        CALLBACKMETHOD_INVALID: Final[int] = -1
        CALLBACKENCODING_FORM: Final[int] = 0
        CALLBACKENCODING_JSON: Final[int] = 1
        CALLBACKENCODING_JSON_ARRAY: Final[int] = 2
        CALLBACKENCODING_CSV: Final[int] = 3
        CALLBACKENCODING_YOCTO_API: Final[int] = 4
        CALLBACKENCODING_JSON_NUM: Final[int] = 5
        CALLBACKENCODING_EMONCMS: Final[int] = 6
        CALLBACKENCODING_AZURE: Final[int] = 7
        CALLBACKENCODING_INFLUXDB: Final[int] = 8
        CALLBACKENCODING_MQTT: Final[int] = 9
        CALLBACKENCODING_YOCTO_API_JZON: Final[int] = 10
        CALLBACKENCODING_PRTG: Final[int] = 11
        CALLBACKENCODING_INFLUXDB_V2: Final[int] = 12
        CALLBACKENCODING_INVALID: Final[int] = -1
        CALLBACKTEMPLATE_OFF: Final[int] = 0
        CALLBACKTEMPLATE_ON: Final[int] = 1
        CALLBACKTEMPLATE_INVALID: Final[int] = -1
        # --- (end of YNetwork return codes)


    # --- (YNetwork implementation)

    @classmethod
    def FirstNetwork(cls) -> Union[YNetwork, None]:
        """
        Starts the enumeration of network interfaces currently accessible.
        Use the method YNetwork.nextNetwork() to iterate on
        next network interfaces.

        @return a pointer to a YNetwork object, corresponding to
                the first network interface currently online, or a None pointer
                if there are none.
        """
        return cls._proxy(cls, YNetwork_aio.FirstNetwork())

    @classmethod
    def FirstNetworkInContext(cls, yctx: YAPIContext) -> Union[YNetwork, None]:
        """
        Starts the enumeration of network interfaces currently accessible.
        Use the method YNetwork.nextNetwork() to iterate on
        next network interfaces.

        @param yctx : a YAPI context.

        @return a pointer to a YNetwork object, corresponding to
                the first network interface currently online, or a None pointer
                if there are none.
        """
        return cls._proxy(cls, YNetwork_aio.FirstNetworkInContext(yctx))

    def nextNetwork(self):
        """
        Continues the enumeration of network interfaces started using yFirstNetwork().
        Caution: You can't make any assumption about the returned network interfaces order.
        If you want to find a specific a network interface, use Network.findNetwork()
        and a hardwareID or a logical name.

        @return a pointer to a YNetwork object, corresponding to
                a network interface currently online, or a None pointer
                if there are no more network interfaces to enumerate.
        """
        return self._proxy(type(self), self._aio.nextNetwork())

    if not _DYNAMIC_HELPERS:
        def get_readiness(self) -> int:
            """
            Returns the current established working mode of the network interface.
            Level zero (DOWN_0) means that no hardware link has been detected. Either there is no signal
            on the network cable, or the selected wireless access point cannot be detected.
            Level 1 (LIVE_1) is reached when the network is detected, but is not yet connected.
            For a wireless network, this shows that the requested SSID is present.
            Level 2 (LINK_2) is reached when the hardware connection is established.
            For a wired network connection, level 2 means that the cable is attached at both ends.
            For a connection to a wireless access point, it shows that the security parameters
            are properly configured. For an ad-hoc wireless connection, it means that there is
            at least one other device connected on the ad-hoc network.
            Level 3 (DHCP_3) is reached when an IP address has been obtained using DHCP.
            Level 4 (DNS_4) is reached when the DNS server is reachable on the network.
            Level 5 (WWW_5) is reached when global connectivity is demonstrated by properly loading the
            current time from an NTP server.

            @return a value among YNetwork.READINESS_DOWN, YNetwork.READINESS_EXISTS,
            YNetwork.READINESS_LINKED, YNetwork.READINESS_LAN_OK and YNetwork.READINESS_WWW_OK corresponding to
            the current established working mode of the network interface

            On failure, throws an exception or returns YNetwork.READINESS_INVALID.
            """
            return self._run(self._aio.get_readiness())

    if not _DYNAMIC_HELPERS:
        def get_macAddress(self) -> str:
            """
            Returns the MAC address of the network interface. The MAC address is also available on a sticker
            on the module, in both numeric and barcode forms.

            @return a string corresponding to the MAC address of the network interface

            On failure, throws an exception or returns YNetwork.MACADDRESS_INVALID.
            """
            return self._run(self._aio.get_macAddress())

    if not _DYNAMIC_HELPERS:
        def get_ipAddress(self) -> str:
            """
            Returns the IP address currently in use by the device. The address may have been configured
            statically, or provided by a DHCP server.

            @return a string corresponding to the IP address currently in use by the device

            On failure, throws an exception or returns YNetwork.IPADDRESS_INVALID.
            """
            return self._run(self._aio.get_ipAddress())

    if not _DYNAMIC_HELPERS:
        def get_subnetMask(self) -> str:
            """
            Returns the subnet mask currently used by the device.

            @return a string corresponding to the subnet mask currently used by the device

            On failure, throws an exception or returns YNetwork.SUBNETMASK_INVALID.
            """
            return self._run(self._aio.get_subnetMask())

    if not _DYNAMIC_HELPERS:
        def get_router(self) -> str:
            """
            Returns the IP address of the router on the device subnet (default gateway).

            @return a string corresponding to the IP address of the router on the device subnet (default gateway)

            On failure, throws an exception or returns YNetwork.ROUTER_INVALID.
            """
            return self._run(self._aio.get_router())

    if not _DYNAMIC_HELPERS:
        def get_currentDNS(self) -> str:
            """
            Returns the IP address of the DNS server currently used by the device.

            @return a string corresponding to the IP address of the DNS server currently used by the device

            On failure, throws an exception or returns YNetwork.CURRENTDNS_INVALID.
            """
            return self._run(self._aio.get_currentDNS())

    if not _DYNAMIC_HELPERS:
        def get_ipConfig(self) -> str:
            """
            Returns the IP configuration of the network interface.

            If the network interface is set up to use a static IP address, the string starts with "STATIC:" and
            is followed by three
            parameters, separated by "/". The first is the device IP address, followed by the subnet mask
            length, and finally the
            router IP address (default gateway). For instance: "STATIC:192.168.1.14/16/192.168.1.1"

            If the network interface is configured to receive its IP from a DHCP server, the string start with
            "DHCP:" and is followed by
            three parameters separated by "/". The first is the fallback IP address, then the fallback subnet
            mask length and finally the
            fallback router IP address. These three parameters are used when no DHCP reply is received.

            @return a string corresponding to the IP configuration of the network interface

            On failure, throws an exception or returns YNetwork.IPCONFIG_INVALID.
            """
            return self._run(self._aio.get_ipConfig())

    if not _DYNAMIC_HELPERS:
        def set_ipConfig(self, newval: str) -> int:
            return self._run(self._aio.set_ipConfig(newval))

    if not _DYNAMIC_HELPERS:
        def get_primaryDNS(self) -> str:
            """
            Returns the IP address of the primary name server to be used by the module.

            @return a string corresponding to the IP address of the primary name server to be used by the module

            On failure, throws an exception or returns YNetwork.PRIMARYDNS_INVALID.
            """
            return self._run(self._aio.get_primaryDNS())

    if not _DYNAMIC_HELPERS:
        def set_primaryDNS(self, newval: str) -> int:
            """
            Changes the IP address of the primary name server to be used by the module.
            When using DHCP, if a value is specified, it overrides the value received from the DHCP server.
            Remember to call the saveToFlash() method and then to reboot the module to apply this setting.

            @param newval : a string corresponding to the IP address of the primary name server to be used by the module

            @return YAPI.SUCCESS if the call succeeds.

            On failure, throws an exception or returns a negative error code.
            """
            return self._run(self._aio.set_primaryDNS(newval))

    if not _DYNAMIC_HELPERS:
        def get_secondaryDNS(self) -> str:
            """
            Returns the IP address of the secondary name server to be used by the module.

            @return a string corresponding to the IP address of the secondary name server to be used by the module

            On failure, throws an exception or returns YNetwork.SECONDARYDNS_INVALID.
            """
            return self._run(self._aio.get_secondaryDNS())

    if not _DYNAMIC_HELPERS:
        def set_secondaryDNS(self, newval: str) -> int:
            """
            Changes the IP address of the secondary name server to be used by the module.
            When using DHCP, if a value is specified, it overrides the value received from the DHCP server.
            Remember to call the saveToFlash() method and then to reboot the module to apply this setting.

            @param newval : a string corresponding to the IP address of the secondary name server to be used by the module

            @return YAPI.SUCCESS if the call succeeds.

            On failure, throws an exception or returns a negative error code.
            """
            return self._run(self._aio.set_secondaryDNS(newval))

    if not _DYNAMIC_HELPERS:
        def get_ntpServer(self) -> str:
            """
            Returns the IP address of the NTP server to be used by the device.

            @return a string corresponding to the IP address of the NTP server to be used by the device

            On failure, throws an exception or returns YNetwork.NTPSERVER_INVALID.
            """
            return self._run(self._aio.get_ntpServer())

    if not _DYNAMIC_HELPERS:
        def set_ntpServer(self, newval: str) -> int:
            """
            Changes the IP address of the NTP server to be used by the module. Use an empty
            string to restore the factory set  address.
            Remember to call the saveToFlash() method and then to reboot the module to apply this setting.

            @param newval : a string corresponding to the IP address of the NTP server to be used by the module

            @return YAPI.SUCCESS if the call succeeds.

            On failure, throws an exception or returns a negative error code.
            """
            return self._run(self._aio.set_ntpServer(newval))

    if not _DYNAMIC_HELPERS:
        def get_userPassword(self) -> str:
            """
            Returns a hash string if a password has been set for "user" user,
            or an empty string otherwise.

            @return a string corresponding to a hash string if a password has been set for "user" user,
                    or an empty string otherwise

            On failure, throws an exception or returns YNetwork.USERPASSWORD_INVALID.
            """
            return self._run(self._aio.get_userPassword())

    if not _DYNAMIC_HELPERS:
        def set_userPassword(self, newval: str) -> int:
            """
            Changes the password for the "user" user. This password becomes instantly required
            to perform any use of the module. If the specified value is an
            empty string, a password is not required anymore.
            Remember to call the saveToFlash() method of the module if the
            modification must be kept.

            @param newval : a string corresponding to the password for the "user" user

            @return YAPI.SUCCESS if the call succeeds.

            On failure, throws an exception or returns a negative error code.
            """
            return self._run(self._aio.set_userPassword(newval))

    if not _DYNAMIC_HELPERS:
        def get_adminPassword(self) -> str:
            """
            Returns a hash string if a password has been set for user "admin",
            or an empty string otherwise.

            @return a string corresponding to a hash string if a password has been set for user "admin",
                    or an empty string otherwise

            On failure, throws an exception or returns YNetwork.ADMINPASSWORD_INVALID.
            """
            return self._run(self._aio.get_adminPassword())

    if not _DYNAMIC_HELPERS:
        def set_adminPassword(self, newval: str) -> int:
            """
            Changes the password for the "admin" user. This password becomes instantly required
            to perform any change of the module state. If the specified value is an
            empty string, a password is not required anymore.
            Remember to call the saveToFlash() method of the module if the
            modification must be kept.

            @param newval : a string corresponding to the password for the "admin" user

            @return YAPI.SUCCESS if the call succeeds.

            On failure, throws an exception or returns a negative error code.
            """
            return self._run(self._aio.set_adminPassword(newval))

    if not _DYNAMIC_HELPERS:
        def get_httpPort(self) -> int:
            """
            Returns the TCP port used to serve the hub web UI.

            @return an integer corresponding to the TCP port used to serve the hub web UI

            On failure, throws an exception or returns YNetwork.HTTPPORT_INVALID.
            """
            return self._run(self._aio.get_httpPort())

    if not _DYNAMIC_HELPERS:
        def set_httpPort(self, newval: int) -> int:
            """
            Changes the the TCP port used to serve the hub web UI. The default value is port 80,
            which is the default for all Web servers. Regardless of the value set here,
            the hub will always reply on port 4444, which is used by default by Yoctopuce
            API library. When you change this parameter, remember to call the saveToFlash()
            method of the module if the modification must be kept.

            @param newval : an integer corresponding to the the TCP port used to serve the hub web UI

            @return YAPI.SUCCESS if the call succeeds.

            On failure, throws an exception or returns a negative error code.
            """
            return self._run(self._aio.set_httpPort(newval))

    if not _DYNAMIC_HELPERS:
        def get_httpsPort(self) -> int:
            """
            Returns the secure TCP port used to serve the hub web UI.

            @return an integer corresponding to the secure TCP port used to serve the hub web UI

            On failure, throws an exception or returns YNetwork.HTTPSPORT_INVALID.
            """
            return self._run(self._aio.get_httpsPort())

    if not _DYNAMIC_HELPERS:
        def set_httpsPort(self, newval: int) -> int:
            """
            Changes the secure TCP port used to serve the hub web UI. The default value is port 4443,
            which is the default for all Web servers. When you change this parameter, remember to call the saveToFlash()
            method of the module if the modification must be kept.

            @param newval : an integer corresponding to the secure TCP port used to serve the hub web UI

            @return YAPI.SUCCESS if the call succeeds.

            On failure, throws an exception or returns a negative error code.
            """
            return self._run(self._aio.set_httpsPort(newval))

    if not _DYNAMIC_HELPERS:
        def get_securityMode(self) -> int:
            """
            Returns the security level chosen to prevent unauthorized access to the server.

            @return a value among YNetwork.SECURITYMODE_UNDEFINED, YNetwork.SECURITYMODE_LEGACY,
            YNetwork.SECURITYMODE_MIXED and YNetwork.SECURITYMODE_SECURE corresponding to the security level
            chosen to prevent unauthorized access to the server

            On failure, throws an exception or returns YNetwork.SECURITYMODE_INVALID.
            """
            return self._run(self._aio.get_securityMode())

    if not _DYNAMIC_HELPERS:
        def set_securityMode(self, newval: int) -> int:
            """
            Changes the security level used to prevent unauthorized access to the server.
            The value UNDEFINED causes the security configuration wizard to be
            displayed the next time you log on to the Web console.
            The value LEGACY offers unencrypted HTTP access by default, and
            is designed to provide compatibility with legacy applications that do not
            handle password or do not support HTTPS. But it should
            only be used when system security is guaranteed by other means, such as the
            use of a firewall.
            The value MIXED requires the configuration of passwords, and allows
            access via both HTTP (unencrypted) and HTTPS (encrypted), while requiring
            the Yoctopuce API to be tolerant of certificate characteristics.
            The value SECURE requires the configuration of passwords and the
            use of secure communications in all cases.
            When you change this parameter, remember to call the saveToFlash()
            method of the module if the modification must be kept.

            @param newval : a value among YNetwork.SECURITYMODE_UNDEFINED, YNetwork.SECURITYMODE_LEGACY,
            YNetwork.SECURITYMODE_MIXED and YNetwork.SECURITYMODE_SECURE corresponding to the security level
            used to prevent unauthorized access to the server

            @return YAPI.SUCCESS if the call succeeds.

            On failure, throws an exception or returns a negative error code.
            """
            return self._run(self._aio.set_securityMode(newval))

    if not _DYNAMIC_HELPERS:
        def get_defaultPage(self) -> str:
            """
            Returns the HTML page to serve for the URL "/"" of the hub.

            @return a string corresponding to the HTML page to serve for the URL "/"" of the hub

            On failure, throws an exception or returns YNetwork.DEFAULTPAGE_INVALID.
            """
            return self._run(self._aio.get_defaultPage())

    if not _DYNAMIC_HELPERS:
        def set_defaultPage(self, newval: str) -> int:
            """
            Changes the default HTML page returned by the hub. If not value are set the hub return
            "index.html" which is the web interface of the hub. It is possible to change this page
            for file that has been uploaded on the hub. The maximum filename size is 15 characters.
            When you change this parameter, remember to call the saveToFlash()
            method of the module if the modification must be kept.

            @param newval : a string corresponding to the default HTML page returned by the hub

            @return YAPI.SUCCESS if the call succeeds.

            On failure, throws an exception or returns a negative error code.
            """
            return self._run(self._aio.set_defaultPage(newval))

    if not _DYNAMIC_HELPERS:
        def get_discoverable(self) -> int:
            """
            Returns the activation state of the multicast announce protocols to allow easy
            discovery of the module in the network neighborhood (uPnP/Bonjour protocol).

            @return either YNetwork.DISCOVERABLE_FALSE or YNetwork.DISCOVERABLE_TRUE, according to the
            activation state of the multicast announce protocols to allow easy
                    discovery of the module in the network neighborhood (uPnP/Bonjour protocol)

            On failure, throws an exception or returns YNetwork.DISCOVERABLE_INVALID.
            """
            return self._run(self._aio.get_discoverable())

    if not _DYNAMIC_HELPERS:
        def set_discoverable(self, newval: int) -> int:
            """
            Changes the activation state of the multicast announce protocols to allow easy
            discovery of the module in the network neighborhood (uPnP/Bonjour protocol).
            Remember to call the saveToFlash()
            method of the module if the modification must be kept.

            @param newval : either YNetwork.DISCOVERABLE_FALSE or YNetwork.DISCOVERABLE_TRUE, according to the
            activation state of the multicast announce protocols to allow easy
                    discovery of the module in the network neighborhood (uPnP/Bonjour protocol)

            @return YAPI.SUCCESS if the call succeeds.

            On failure, throws an exception or returns a negative error code.
            """
            return self._run(self._aio.set_discoverable(newval))

    if not _DYNAMIC_HELPERS:
        def get_wwwWatchdogDelay(self) -> int:
            """
            Returns the allowed downtime of the WWW link (in seconds) before triggering an automated
            reboot to try to recover Internet connectivity. A zero value disables automated reboot
            in case of Internet connectivity loss.

            @return an integer corresponding to the allowed downtime of the WWW link (in seconds) before
            triggering an automated
                    reboot to try to recover Internet connectivity

            On failure, throws an exception or returns YNetwork.WWWWATCHDOGDELAY_INVALID.
            """
            return self._run(self._aio.get_wwwWatchdogDelay())

    if not _DYNAMIC_HELPERS:
        def set_wwwWatchdogDelay(self, newval: int) -> int:
            """
            Changes the allowed downtime of the WWW link (in seconds) before triggering an automated
            reboot to try to recover Internet connectivity. A zero value disables automated reboot
            in case of Internet connectivity loss. The smallest valid non-zero timeout is
            90 seconds. Remember to call the saveToFlash()
            method of the module if the modification must be kept.

            @param newval : an integer corresponding to the allowed downtime of the WWW link (in seconds)
            before triggering an automated
                    reboot to try to recover Internet connectivity

            @return YAPI.SUCCESS if the call succeeds.

            On failure, throws an exception or returns a negative error code.
            """
            return self._run(self._aio.set_wwwWatchdogDelay(newval))

    if not _DYNAMIC_HELPERS:
        def get_callbackUrl(self) -> str:
            """
            Returns the callback URL to notify of significant state changes.

            @return a string corresponding to the callback URL to notify of significant state changes

            On failure, throws an exception or returns YNetwork.CALLBACKURL_INVALID.
            """
            return self._run(self._aio.get_callbackUrl())

    if not _DYNAMIC_HELPERS:
        def set_callbackUrl(self, newval: str) -> int:
            """
            Changes the callback URL to notify significant state changes. Remember to call the
            saveToFlash() method of the module if the modification must be kept.

            @param newval : a string corresponding to the callback URL to notify significant state changes

            @return YAPI.SUCCESS if the call succeeds.

            On failure, throws an exception or returns a negative error code.
            """
            return self._run(self._aio.set_callbackUrl(newval))

    if not _DYNAMIC_HELPERS:
        def get_callbackMethod(self) -> int:
            """
            Returns the HTTP method used to notify callbacks for significant state changes.

            @return a value among YNetwork.CALLBACKMETHOD_POST, YNetwork.CALLBACKMETHOD_GET and
            YNetwork.CALLBACKMETHOD_PUT corresponding to the HTTP method used to notify callbacks for
            significant state changes

            On failure, throws an exception or returns YNetwork.CALLBACKMETHOD_INVALID.
            """
            return self._run(self._aio.get_callbackMethod())

    if not _DYNAMIC_HELPERS:
        def set_callbackMethod(self, newval: int) -> int:
            """
            Changes the HTTP method used to notify callbacks for significant state changes.
            Remember to call the saveToFlash() method of the module if the
            modification must be kept.

            @param newval : a value among YNetwork.CALLBACKMETHOD_POST, YNetwork.CALLBACKMETHOD_GET and
            YNetwork.CALLBACKMETHOD_PUT corresponding to the HTTP method used to notify callbacks for
            significant state changes

            @return YAPI.SUCCESS if the call succeeds.

            On failure, throws an exception or returns a negative error code.
            """
            return self._run(self._aio.set_callbackMethod(newval))

    if not _DYNAMIC_HELPERS:
        def get_callbackEncoding(self) -> int:
            """
            Returns the encoding standard to use for representing notification values.

            @return a value among YNetwork.CALLBACKENCODING_FORM, YNetwork.CALLBACKENCODING_JSON,
            YNetwork.CALLBACKENCODING_JSON_ARRAY, YNetwork.CALLBACKENCODING_CSV,
            YNetwork.CALLBACKENCODING_YOCTO_API, YNetwork.CALLBACKENCODING_JSON_NUM,
            YNetwork.CALLBACKENCODING_EMONCMS, YNetwork.CALLBACKENCODING_AZURE,
            YNetwork.CALLBACKENCODING_INFLUXDB, YNetwork.CALLBACKENCODING_MQTT,
            YNetwork.CALLBACKENCODING_YOCTO_API_JZON, YNetwork.CALLBACKENCODING_PRTG and
            YNetwork.CALLBACKENCODING_INFLUXDB_V2 corresponding to the encoding standard to use for
            representing notification values

            On failure, throws an exception or returns YNetwork.CALLBACKENCODING_INVALID.
            """
            return self._run(self._aio.get_callbackEncoding())

    if not _DYNAMIC_HELPERS:
        def set_callbackEncoding(self, newval: int) -> int:
            """
            Changes the encoding standard to use for representing notification values.
            Remember to call the saveToFlash() method of the module if the
            modification must be kept.

            @param newval : a value among YNetwork.CALLBACKENCODING_FORM, YNetwork.CALLBACKENCODING_JSON,
            YNetwork.CALLBACKENCODING_JSON_ARRAY, YNetwork.CALLBACKENCODING_CSV,
            YNetwork.CALLBACKENCODING_YOCTO_API, YNetwork.CALLBACKENCODING_JSON_NUM,
            YNetwork.CALLBACKENCODING_EMONCMS, YNetwork.CALLBACKENCODING_AZURE,
            YNetwork.CALLBACKENCODING_INFLUXDB, YNetwork.CALLBACKENCODING_MQTT,
            YNetwork.CALLBACKENCODING_YOCTO_API_JZON, YNetwork.CALLBACKENCODING_PRTG and
            YNetwork.CALLBACKENCODING_INFLUXDB_V2 corresponding to the encoding standard to use for
            representing notification values

            @return YAPI.SUCCESS if the call succeeds.

            On failure, throws an exception or returns a negative error code.
            """
            return self._run(self._aio.set_callbackEncoding(newval))

    if not _DYNAMIC_HELPERS:
        def get_callbackTemplate(self) -> int:
            """
            Returns the activation state of the custom template file to customize callback
            format. If the custom callback template is disabled, it will be ignored even
            if present on the YoctoHub.

            @return either YNetwork.CALLBACKTEMPLATE_OFF or YNetwork.CALLBACKTEMPLATE_ON, according to the
            activation state of the custom template file to customize callback
                    format

            On failure, throws an exception or returns YNetwork.CALLBACKTEMPLATE_INVALID.
            """
            return self._run(self._aio.get_callbackTemplate())

    if not _DYNAMIC_HELPERS:
        def set_callbackTemplate(self, newval: int) -> int:
            """
            Enable the use of a template file to customize callbacks format.
            When the custom callback template file is enabled, the template file
            will be loaded for each callback in order to build the data to post to the
            server. If template file does not exist on the YoctoHub, the callback will
            fail with an error message indicating the name of the expected template file.
            Remember to call the saveToFlash() method of the module if the
            modification must be kept.

            @param newval : either YNetwork.CALLBACKTEMPLATE_OFF or YNetwork.CALLBACKTEMPLATE_ON

            @return YAPI.SUCCESS if the call succeeds.

            On failure, throws an exception or returns a negative error code.
            """
            return self._run(self._aio.set_callbackTemplate(newval))

    if not _DYNAMIC_HELPERS:
        def get_callbackCredentials(self) -> str:
            """
            Returns a hashed version of the notification callback credentials if set,
            or an empty string otherwise.

            @return a string corresponding to a hashed version of the notification callback credentials if set,
                    or an empty string otherwise

            On failure, throws an exception or returns YNetwork.CALLBACKCREDENTIALS_INVALID.
            """
            return self._run(self._aio.get_callbackCredentials())

    if not _DYNAMIC_HELPERS:
        def set_callbackCredentials(self, newval: str) -> int:
            """
            Changes the credentials required to connect to the callback address. The credentials
            must be provided as returned by function get_callbackCredentials,
            in the form username:hash. The method used to compute the hash varies according
            to the the authentication scheme implemented by the callback, For Basic authentication,
            the hash is the MD5 of the string username:password. For Digest authentication,
            the hash is the MD5 of the string username:realm:password. For a simpler
            way to configure callback credentials, use function callbackLogin instead.
            Remember to call the saveToFlash() method of the module if the
            modification must be kept.

            @param newval : a string corresponding to the credentials required to connect to the callback address

            @return YAPI.SUCCESS if the call succeeds.

            On failure, throws an exception or returns a negative error code.
            """
            return self._run(self._aio.set_callbackCredentials(newval))

    if not _DYNAMIC_HELPERS:
        def callbackLogin(self, username, password) -> int:
            """
            Connects to the notification callback and saves the credentials required to
            log into it. The password is not stored into the module, only a hashed
            copy of the credentials are saved. Remember to call the
            saveToFlash() method of the module if the modification must be kept.

            @param username : username required to log to the callback
            @param password : password required to log to the callback

            @return YAPI.SUCCESS if the call succeeds.

            On failure, throws an exception or returns a negative error code.
            """
            return self._run(self._aio.callbackLogin(username, password))

    if not _DYNAMIC_HELPERS:
        def get_callbackInitialDelay(self) -> int:
            """
            Returns the initial waiting time before first callback notifications, in seconds.

            @return an integer corresponding to the initial waiting time before first callback notifications, in seconds

            On failure, throws an exception or returns YNetwork.CALLBACKINITIALDELAY_INVALID.
            """
            return self._run(self._aio.get_callbackInitialDelay())

    if not _DYNAMIC_HELPERS:
        def set_callbackInitialDelay(self, newval: int) -> int:
            """
            Changes the initial waiting time before first callback notifications, in seconds.
            Remember to call the saveToFlash() method of the module if the modification must be kept.

            @param newval : an integer corresponding to the initial waiting time before first callback
            notifications, in seconds

            @return YAPI.SUCCESS if the call succeeds.

            On failure, throws an exception or returns a negative error code.
            """
            return self._run(self._aio.set_callbackInitialDelay(newval))

    if not _DYNAMIC_HELPERS:
        def get_callbackSchedule(self) -> str:
            """
            Returns the HTTP callback schedule strategy, as a text string.

            @return a string corresponding to the HTTP callback schedule strategy, as a text string

            On failure, throws an exception or returns YNetwork.CALLBACKSCHEDULE_INVALID.
            """
            return self._run(self._aio.get_callbackSchedule())

    if not _DYNAMIC_HELPERS:
        def set_callbackSchedule(self, newval: str) -> int:
            """
            Changes the HTTP callback schedule strategy, as a text string.
            Remember to call the saveToFlash()
            method of the module if the modification must be kept.

            @param newval : a string corresponding to the HTTP callback schedule strategy, as a text string

            @return YAPI.SUCCESS if the call succeeds.

            On failure, throws an exception or returns a negative error code.
            """
            return self._run(self._aio.set_callbackSchedule(newval))

    if not _DYNAMIC_HELPERS:
        def get_callbackMinDelay(self) -> int:
            """
            Returns the minimum waiting time between two HTTP callbacks, in seconds.

            @return an integer corresponding to the minimum waiting time between two HTTP callbacks, in seconds

            On failure, throws an exception or returns YNetwork.CALLBACKMINDELAY_INVALID.
            """
            return self._run(self._aio.get_callbackMinDelay())

    if not _DYNAMIC_HELPERS:
        def set_callbackMinDelay(self, newval: int) -> int:
            """
            Changes the minimum waiting time between two HTTP callbacks, in seconds.
            Remember to call the saveToFlash() method of the module if the modification must be kept.

            @param newval : an integer corresponding to the minimum waiting time between two HTTP callbacks, in seconds

            @return YAPI.SUCCESS if the call succeeds.

            On failure, throws an exception or returns a negative error code.
            """
            return self._run(self._aio.set_callbackMinDelay(newval))

    if not _DYNAMIC_HELPERS:
        def get_callbackMaxDelay(self) -> int:
            """
            Returns the waiting time between two HTTP callbacks when there is nothing new.

            @return an integer corresponding to the waiting time between two HTTP callbacks when there is nothing new

            On failure, throws an exception or returns YNetwork.CALLBACKMAXDELAY_INVALID.
            """
            return self._run(self._aio.get_callbackMaxDelay())

    if not _DYNAMIC_HELPERS:
        def set_callbackMaxDelay(self, newval: int) -> int:
            """
            Changes the waiting time between two HTTP callbacks when there is nothing new.
            Remember to call the saveToFlash() method of the module if the modification must be kept.

            @param newval : an integer corresponding to the waiting time between two HTTP callbacks when there
            is nothing new

            @return YAPI.SUCCESS if the call succeeds.

            On failure, throws an exception or returns a negative error code.
            """
            return self._run(self._aio.set_callbackMaxDelay(newval))

    if not _DYNAMIC_HELPERS:
        def get_poeCurrent(self) -> int:
            """
            Returns the current consumed by the module from Power-over-Ethernet (PoE), in milliamps.
            The current consumption is measured after converting PoE source to 5 Volt, and should
            never exceed 1800 mA.

            @return an integer corresponding to the current consumed by the module from Power-over-Ethernet
            (PoE), in milliamps

            On failure, throws an exception or returns YNetwork.POECURRENT_INVALID.
            """
            return self._run(self._aio.get_poeCurrent())

    @classmethod
    def FindNetwork(cls, func: str) -> YNetwork:
        """
        Retrieves a network interface for a given identifier.
        The identifier can be specified using several formats:

        - FunctionLogicalName
        - ModuleSerialNumber.FunctionIdentifier
        - ModuleSerialNumber.FunctionLogicalName
        - ModuleLogicalName.FunctionIdentifier
        - ModuleLogicalName.FunctionLogicalName


        This function does not require that the network interface is online at the time
        it is invoked. The returned object is nevertheless valid.
        Use the method YNetwork.isOnline() to test if the network interface is
        indeed online at a given time. In case of ambiguity when looking for
        a network interface by logical name, no error is notified: the first instance
        found is returned. The search is performed first by hardware name,
        then by logical name.

        If a call to this object's is_online() method returns FALSE although
        you are certain that the matching device is plugged, make sure that you did
        call registerHub() at application initialization time.

        @param func : a string that uniquely characterizes the network interface, for instance
                YHUBETH1.network.

        @return a YNetwork object allowing you to drive the network interface.
        """
        return cls._proxy(cls, YNetwork_aio.FindNetwork(func))

    @classmethod
    def FindNetworkInContext(cls, yctx: YAPIContext, func: str) -> YNetwork:
        """
        Retrieves a network interface for a given identifier in a YAPI context.
        The identifier can be specified using several formats:

        - FunctionLogicalName
        - ModuleSerialNumber.FunctionIdentifier
        - ModuleSerialNumber.FunctionLogicalName
        - ModuleLogicalName.FunctionIdentifier
        - ModuleLogicalName.FunctionLogicalName


        This function does not require that the network interface is online at the time
        it is invoked. The returned object is nevertheless valid.
        Use the method YNetwork.isOnline() to test if the network interface is
        indeed online at a given time. In case of ambiguity when looking for
        a network interface by logical name, no error is notified: the first instance
        found is returned. The search is performed first by hardware name,
        then by logical name.

        @param yctx : a YAPI context
        @param func : a string that uniquely characterizes the network interface, for instance
                YHUBETH1.network.

        @return a YNetwork object allowing you to drive the network interface.
        """
        return cls._proxy(cls, YNetwork_aio.FindNetworkInContext(yctx, func))

    if not _IS_MICROPYTHON:
        def registerValueCallback(self, callback: YNetworkValueCallback) -> int:
            """
            Registers the callback function that is invoked on every change of advertised value.
            The callback is invoked only during the execution of ySleep or yHandleEvents.
            This provides control over the time when the callback is triggered. For good responsiveness, remember to call
            one of these two functions periodically. To unregister a callback, pass a None pointer as argument.

            @param callback : the callback function to call, or a None pointer. The callback function should take two
                    arguments: the function object of which the value has changed, and the character string describing
                    the new advertised value.
            @noreturn
            """
            return super().registerValueCallback(callback)

    if not _DYNAMIC_HELPERS:
        def useDHCP(self, fallbackIpAddr: str, fallbackSubnetMaskLen: int, fallbackRouter: str) -> int:
            """
            Changes the configuration of the network interface to enable the use of an
            IP address received from a DHCP server. Until an address is received from a DHCP
            server, the module uses the IP parameters specified to this function.
            Remember to call the saveToFlash() method and then to reboot the module to apply this setting.

            @param fallbackIpAddr : fallback IP address, to be used when no DHCP reply is received
            @param fallbackSubnetMaskLen : fallback subnet mask length when no DHCP reply is received, as an
                    integer (e.g. 24 means 255.255.255.0)
            @param fallbackRouter : fallback router IP address, to be used when no DHCP reply is received

            @return YAPI.SUCCESS when the call succeeds.

            On failure, throws an exception or returns a negative error code.
            """
            return self._run(self._aio.useDHCP(fallbackIpAddr, fallbackSubnetMaskLen, fallbackRouter))

    if not _DYNAMIC_HELPERS:
        def useDHCPauto(self) -> int:
            """
            Changes the configuration of the network interface to enable the use of an
            IP address received from a DHCP server. Until an address is received from a DHCP
            server, the module uses an IP of the network 169.254.0.0/16 (APIPA).
            Remember to call the saveToFlash() method and then to reboot the module to apply this setting.

            @return YAPI.SUCCESS when the call succeeds.

            On failure, throws an exception or returns a negative error code.
            """
            return self._run(self._aio.useDHCPauto())

    if not _DYNAMIC_HELPERS:
        def useStaticIP(self, ipAddress: str, subnetMaskLen: int, router: str) -> int:
            """
            Changes the configuration of the network interface to use a static IP address.
            Remember to call the saveToFlash() method and then to reboot the module to apply this setting.

            @param ipAddress : device IP address
            @param subnetMaskLen : subnet mask length, as an integer (e.g. 24 means 255.255.255.0)
            @param router : router IP address (default gateway)

            @return YAPI.SUCCESS when the call succeeds.

            On failure, throws an exception or returns a negative error code.
            """
            return self._run(self._aio.useStaticIP(ipAddress, subnetMaskLen, router))

    if not _DYNAMIC_HELPERS:
        def ping(self, host: str) -> str:
            """
            Pings host to test the network connectivity. Sends four ICMP ECHO_REQUEST requests from the
            module to the target host. This method returns a string with the result of the
            4 ICMP ECHO_REQUEST requests.

            @param host : the hostname or the IP address of the target

            @return a string with the result of the ping.
            """
            return self._run(self._aio.ping(host))

    if not _DYNAMIC_HELPERS:
        def triggerCallback(self) -> int:
            """
            Trigger an HTTP callback quickly. This function can even be called within
            an HTTP callback, in which case the next callback will be triggered 5 seconds
            after the end of the current callback, regardless if the minimum time between
            callbacks configured in the device.

            @return YAPI.SUCCESS when the call succeeds.

            On failure, throws an exception or returns a negative error code.
            """
            return self._run(self._aio.triggerCallback())

    if not _DYNAMIC_HELPERS:
        def set_periodicCallbackSchedule(self, interval: str, offset: int) -> int:
            """
            Set up periodic HTTP callbacks (simplified function).

            @param interval : a string representing the callback periodicity, expressed in
                    seconds, minutes or hours, eg. "60s", "5m", "1h", "48h".
            @param offset : an integer representing the time offset relative to the period
                    when the callback should occur. For instance, if the periodicity is
                    24h, an offset of 7 will make the callback occur each day at 7AM.

            @return YAPI.SUCCESS when the call succeeds.

            On failure, throws an exception or returns a negative error code.
            """
            return self._run(self._aio.set_periodicCallbackSchedule(interval, offset))

    # --- (end of YNetwork implementation)




# -*- coding: utf-8 -*-
# ********************************************************************
#
#  $Id: svn_id $
#
#  Asyncio implementation of YArithmeticSensor
#
#  - - - - - - - - - License information: - - - - - - - - -
#
#  Copyright (C) 2011 and beyond by Yoctopuce Sarl, Switzerland.
#
#  Yoctopuce Sarl (hereafter Licensor) grants to you a perpetual
#  non-exclusive license to use, modify, copy and integrate this
#  file into your software for the sole purpose of interfacing
#  with Yoctopuce products.
#
#  You may reproduce and distribute copies of this file in
#  source or object form, as long as the sole purpose of this
#  code is to interface with Yoctopuce products. You must retain
#  this notice in the distributed source file.
#
#  You should refer to Yoctopuce General Terms and Conditions
#  for additional information regarding your rights and
#  obligations.
#
#  THE SOFTWARE AND DOCUMENTATION ARE PROVIDED 'AS IS' WITHOUT
#  WARRANTY OF ANY KIND, EITHER EXPRESS OR IMPLIED, INCLUDING
#  WITHOUT LIMITATION, ANY WARRANTY OF MERCHANTABILITY, FITNESS
#  FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO
#  EVENT SHALL LICENSOR BE LIABLE FOR ANY INCIDENTAL, SPECIAL,
#  INDIRECT OR CONSEQUENTIAL DAMAGES, LOST PROFITS OR LOST DATA,
#  COST OF PROCUREMENT OF SUBSTITUTE GOODS, TECHNOLOGY OR
#  SERVICES, ANY CLAIMS BY THIRD PARTIES (INCLUDING BUT NOT
#  LIMITED TO ANY DEFENSE THEREOF), ANY CLAIMS FOR INDEMNITY OR
#  CONTRIBUTION, OR OTHER SIMILAR COSTS, WHETHER ASSERTED ON THE
#  BASIS OF CONTRACT, TORT (INCLUDING NEGLIGENCE), BREACH OF
#  WARRANTY, OR OTHERWISE.
#
# *********************************************************************
"""
Yoctopuce library: Asyncio implementation of YArithmeticSensor
version: PATCH_WITH_VERSION
requires: yocto_api_aio
"""
from __future__ import annotations

import sys

# On MicroPython, code below will be wiped out at compile time
if sys.implementation.name != "micropython":
    # In CPython, enable edit-time type checking, including Final declaration
    from typing import Any, Union, Final
    from collections.abc import Callable, Awaitable
    from .yocto_api_aio import const, _IS_MICROPYTHON
else:
    # In our micropython VM, common generic types are global built-ins
    # Others such as TypeVar should be avoided when using micropython,
    # as they produce overhead in runtime code
    # Final is translated into const() expressions before compilation
    _IS_MICROPYTHON: Final[bool] = True # noqa

from .yocto_api_aio import (
    YAPIContext, YAPI, YAPI_Exception, HwId, hwid2str, YFunction, YSensor, YMeasure, xarray, xbytearray
)

# --- (YArithmeticSensor class start)
if not _IS_MICROPYTHON:
    # For CPython, use strongly typed callback types
    try:
        YArithmeticSensorValueCallback = Union[Callable[['YArithmeticSensor', str], Awaitable[None]], None]
        YArithmeticSensorTimedReportCallback = Union[Callable[['YArithmeticSensor', YMeasure], Awaitable[None]], None]
    except TypeError:
        YArithmeticSensorValueCallback = Union[Callable, Awaitable]
        YArithmeticSensorTimedReportCallback = Union[Callable, Awaitable]

# noinspection PyProtectedMember
class YArithmeticSensor(YSensor):
    """
    The YArithmeticSensor class allows some Yoctopuce devices to compute in real-time
    values based on an arithmetic formula involving one or more measured signals as
    well as the temperature. As for any physical sensor, the computed values can be
    read by callback and stored in the built-in datalogger.

    """
    # --- (end of YArithmeticSensor class start)
    if not _IS_MICROPYTHON:
        # --- (YArithmeticSensor return codes)
        DESCRIPTION_INVALID: Final[str] = YAPI.INVALID_STRING
        COMMAND_INVALID: Final[str] = YAPI.INVALID_STRING
        # --- (end of YArithmeticSensor return codes)

    # --- (YArithmeticSensor attributes declaration)
    _description: str
    _command: str
    _valueCallback: YArithmeticSensorValueCallback
    _timedReportCallback: YArithmeticSensorTimedReportCallback
    # --- (end of YArithmeticSensor attributes declaration)


    def __init__(self, yctx: YAPIContext, func: str):
        super().__init__(yctx, func)
        self._className = 'ArithmeticSensor'
        # --- (YArithmeticSensor constructor)
        self._description = YArithmeticSensor.DESCRIPTION_INVALID
        self._command = YArithmeticSensor.COMMAND_INVALID
        # --- (end of YArithmeticSensor constructor)

    # --- (YArithmeticSensor implementation)

    @staticmethod
    def FirstArithmeticSensor() -> Union[YArithmeticSensor, None]:
        """
        Starts the enumeration of arithmetic sensors currently accessible.
        Use the method YArithmeticSensor.nextArithmeticSensor() to iterate on
        next arithmetic sensors.

        @return a pointer to a YArithmeticSensor object, corresponding to
                the first arithmetic sensor currently online, or a None pointer
                if there are none.
        """
        next_hwid: Union[HwId, None] = YAPI._yHash.getFirstHardwareId('ArithmeticSensor')
        if not next_hwid:
            return None
        return YArithmeticSensor.FindArithmeticSensor(hwid2str(next_hwid))

    @staticmethod
    def FirstArithmeticSensorInContext(yctx: YAPIContext) -> Union[YArithmeticSensor, None]:
        """
        Starts the enumeration of arithmetic sensors currently accessible.
        Use the method YArithmeticSensor.nextArithmeticSensor() to iterate on
        next arithmetic sensors.

        @param yctx : a YAPI context.

        @return a pointer to a YArithmeticSensor object, corresponding to
                the first arithmetic sensor currently online, or a None pointer
                if there are none.
        """
        next_hwid: Union[HwId, None] = yctx._yHash.getFirstHardwareId('ArithmeticSensor')
        if not next_hwid:
            return None
        return YArithmeticSensor.FindArithmeticSensorInContext(yctx, hwid2str(next_hwid))

    def nextArithmeticSensor(self):
        """
        Continues the enumeration of arithmetic sensors started using yFirstArithmeticSensor().
        Caution: You can't make any assumption about the returned arithmetic sensors order.
        If you want to find a specific an arithmetic sensor, use ArithmeticSensor.findArithmeticSensor()
        and a hardwareID or a logical name.

        @return a pointer to a YArithmeticSensor object, corresponding to
                an arithmetic sensor currently online, or a None pointer
                if there are no more arithmetic sensors to enumerate.
        """
        next_hwid: Union[HwId, None] = None
        try:
            hwid: HwId = self._yapi._yHash.resolveHwID(self._className, self._func)
            next_hwid = self._yapi._yHash.getNextHardwareId(self._className, hwid)
        except YAPI_Exception:
            pass
        if not next_hwid:
            return None
        return YArithmeticSensor.FindArithmeticSensorInContext(self._yapi, hwid2str(next_hwid))

    def _parseAttr(self, json_val: dict) -> None:
        if 'description' in json_val:
            self._description = json_val["description"]
        if 'command' in json_val:
            self._command = json_val["command"]
        super()._parseAttr(json_val)

    async def set_unit(self, newval: str) -> int:
        """
        Changes the measuring unit for the arithmetic sensor.
        Remember to call the saveToFlash() method of the module if the
        modification must be kept.

        @param newval : a string corresponding to the measuring unit for the arithmetic sensor

        @return YAPI.SUCCESS if the call succeeds.

        On failure, throws an exception or returns a negative error code.
        """
        rest_val = newval
        return await self._setAttr("unit", rest_val)

    async def get_description(self) -> str:
        """
        Returns a short informative description of the formula.

        @return a string corresponding to a short informative description of the formula

        On failure, throws an exception or returns YArithmeticSensor.DESCRIPTION_INVALID.
        """
        res: str
        if self._cacheExpiration <= YAPI.GetTickCount():
            if await self.load(self._yapi.GetCacheValidity()) != YAPI.SUCCESS:
                return YArithmeticSensor.DESCRIPTION_INVALID
        res = self._description
        return res

    async def get_command(self) -> str:
        res: str
        if self._cacheExpiration <= YAPI.GetTickCount():
            if await self.load(self._yapi.GetCacheValidity()) != YAPI.SUCCESS:
                return YArithmeticSensor.COMMAND_INVALID
        res = self._command
        return res

    async def set_command(self, newval: str) -> int:
        rest_val = newval
        return await self._setAttr("command", rest_val)

    @staticmethod
    def FindArithmeticSensor(func: str) -> YArithmeticSensor:
        """
        Retrieves an arithmetic sensor for a given identifier.
        The identifier can be specified using several formats:

        - FunctionLogicalName
        - ModuleSerialNumber.FunctionIdentifier
        - ModuleSerialNumber.FunctionLogicalName
        - ModuleLogicalName.FunctionIdentifier
        - ModuleLogicalName.FunctionLogicalName


        This function does not require that the arithmetic sensor is online at the time
        it is invoked. The returned object is nevertheless valid.
        Use the method YArithmeticSensor.isOnline() to test if the arithmetic sensor is
        indeed online at a given time. In case of ambiguity when looking for
        an arithmetic sensor by logical name, no error is notified: the first instance
        found is returned. The search is performed first by hardware name,
        then by logical name.

        If a call to this object's is_online() method returns FALSE although
        you are certain that the matching device is plugged, make sure that you did
        call registerHub() at application initialization time.

        @param func : a string that uniquely characterizes the arithmetic sensor, for instance
                RXUVOLT1.arithmeticSensor1.

        @return a YArithmeticSensor object allowing you to drive the arithmetic sensor.
        """
        obj: Union[YArithmeticSensor, None]
        obj = YFunction._FindFromCache("ArithmeticSensor", func)
        if obj is None:
            obj = YArithmeticSensor(YAPI, func)
            YFunction._AddToCache("ArithmeticSensor", func, obj)
        return obj

    @staticmethod
    def FindArithmeticSensorInContext(yctx: YAPIContext, func: str) -> YArithmeticSensor:
        """
        Retrieves an arithmetic sensor for a given identifier in a YAPI context.
        The identifier can be specified using several formats:

        - FunctionLogicalName
        - ModuleSerialNumber.FunctionIdentifier
        - ModuleSerialNumber.FunctionLogicalName
        - ModuleLogicalName.FunctionIdentifier
        - ModuleLogicalName.FunctionLogicalName


        This function does not require that the arithmetic sensor is online at the time
        it is invoked. The returned object is nevertheless valid.
        Use the method YArithmeticSensor.isOnline() to test if the arithmetic sensor is
        indeed online at a given time. In case of ambiguity when looking for
        an arithmetic sensor by logical name, no error is notified: the first instance
        found is returned. The search is performed first by hardware name,
        then by logical name.

        @param yctx : a YAPI context
        @param func : a string that uniquely characterizes the arithmetic sensor, for instance
                RXUVOLT1.arithmeticSensor1.

        @return a YArithmeticSensor object allowing you to drive the arithmetic sensor.
        """
        obj: Union[YArithmeticSensor, None]
        obj = YFunction._FindFromCacheInContext(yctx, "ArithmeticSensor", func)
        if obj is None:
            obj = YArithmeticSensor(yctx, func)
            YFunction._AddToCache("ArithmeticSensor", func, obj)
        return obj

    if not _IS_MICROPYTHON:
        async def registerValueCallback(self, callback: YArithmeticSensorValueCallback) -> int:
            """
            Registers the callback function that is invoked on every change of advertised value.
            The callback is invoked only during the execution of ySleep or yHandleEvents.
            This provides control over the time when the callback is triggered. For good responsiveness, remember to call
            one of these two functions periodically. To unregister a callback, pass a None pointer as argument.

            @param callback : the callback function to call, or a None pointer. The callback function should take two
                    arguments: the function object of which the value has changed, and the character string describing
                    the new advertised value.
            @noreturn
            """
            return await super().registerValueCallback(callback)

    if not _IS_MICROPYTHON:
        async def registerTimedReportCallback(self, callback: YArithmeticSensorTimedReportCallback) -> int:
            """
            Registers the callback function that is invoked on every periodic timed notification.
            The callback is invoked only during the execution of ySleep or yHandleEvents.
            This provides control over the time when the callback is triggered. For good responsiveness, remember to call
            one of these two functions periodically. To unregister a callback, pass a None pointer as argument.

            @param callback : the callback function to call, or a None pointer. The callback function should take two
                    arguments: the function object of which the value has changed, and an YMeasure object describing
                    the new advertised value.
            @noreturn
            """
            return await super().registerTimedReportCallback(callback)

    async def defineExpression(self, expr: str, descr: str) -> float:
        """
        Defines the arithmetic function by means of an algebraic expression. The expression
        may include references to device sensors, by their physical or logical name, to
        usual math functions and to auxiliary functions defined separately.

        @param expr : the algebraic expression defining the function.
        @param descr : short informative description of the expression.

        @return the current expression value if the call succeeds.

        On failure, throws an exception or returns YAPI.INVALID_DOUBLE.
        """
        id: str
        fname: str
        content: str
        data: xarray
        diags: str
        resval: float
        id = await self.get_functionId()
        id = id[16: 16 + len(id) - 16]
        fname = "arithmExpr%s.txt" % id

        content = "// %s\n%s" % (descr, expr)
        data = await self._uploadEx(fname, xbytearray(content, 'latin-1'))
        diags = data.decode('latin-1')
        if not (diags[0: 0 + 8] == "Result: "):
            self._throw(YAPI.INVALID_ARGUMENT, diags)
            return YAPI.INVALID_DOUBLE
        resval = YAPI._atof(diags[8: 8 + len(diags)-8])
        return resval

    async def loadExpression(self) -> str:
        """
        Retrieves the algebraic expression defining the arithmetic function, as previously
        configured using the defineExpression function.

        @return a string containing the mathematical expression.

        On failure, throws an exception or returns a negative error code.
        """
        id: str
        fname: str
        content: str
        idx: int
        id = await self.get_functionId()
        id = id[16: 16 + len(id) - 16]
        fname = "arithmExpr%s.txt" % id

        content = (await self._download(fname)).decode('latin-1')
        idx = content.find("\n")
        if idx > 0:
            content = content[idx+1: idx+1 + len(content)-(idx+1)]
        return content

    async def defineAuxiliaryFunction(self, name: str, inputValues: list[float], outputValues: list[float]) -> int:
        """
        Defines a auxiliary function by means of a table of reference points. Intermediate values
        will be interpolated between specified reference points. The reference points are given
        as pairs of floating point numbers.
        The auxiliary function will be available for use by all ArithmeticSensor objects of the
        device. Up to nine auxiliary function can be defined in a device, each containing up to
        96 reference points.

        @param name : auxiliary function name, up to 16 characters.
        @param inputValues : array of floating point numbers, corresponding to the function input value.
        @param outputValues : array of floating point numbers, corresponding to the output value
                desired for each of the input value, index by index.

        @return YAPI.SUCCESS if the call succeeds.

        On failure, throws an exception or returns a negative error code.
        """
        siz: int
        defstr: str
        idx: int
        inputVal: float
        outputVal: float
        fname: str
        siz = len(inputValues)
        if not (siz > 1):
            self._throw(YAPI.INVALID_ARGUMENT, "auxiliary function must be defined by at least two points")
            return YAPI.INVALID_ARGUMENT
        if not (siz == len(outputValues)):
            self._throw(YAPI.INVALID_ARGUMENT, "table sizes mismatch")
            return YAPI.INVALID_ARGUMENT
        defstr = ""
        idx = 0
        while idx < siz:
            inputVal = inputValues[idx]
            outputVal = outputValues[idx]
            defstr = "%s%f:%f\n" % (defstr, inputVal, outputVal)
            idx = idx + 1
        fname = "userMap%s.txt" % name

        return await self._upload(fname, xbytearray(defstr, 'latin-1'))

    async def loadAuxiliaryFunction(self, name: str, inputValues: list[float], outputValues: list[float]) -> int:
        """
        Retrieves the reference points table defining an auxiliary function previously
        configured using the defineAuxiliaryFunction function.

        @param name : auxiliary function name, up to 16 characters.
        @param inputValues : array of floating point numbers, that is filled by the function
                with all the function reference input value.
        @param outputValues : array of floating point numbers, that is filled by the function
                output value for each of the input value, index by index.

        @return YAPI.SUCCESS if the call succeeds.

        On failure, throws an exception or returns a negative error code.
        """
        fname: str
        defbin: xarray
        siz: int

        fname = "userMap%s.txt" % name
        defbin = await self._download(fname)
        siz = len(defbin)
        if not (siz > 0):
            self._throw(YAPI.INVALID_ARGUMENT, "auxiliary function does not exist")
            return YAPI.INVALID_ARGUMENT
        del inputValues[:]
        del outputValues[:]
        # FIXME: decode line by line
        return YAPI.SUCCESS

    # --- (end of YArithmeticSensor implementation)

